/*
 * DictionaryManager.java
 *
 * Created on 13 de julio de 2007, 8:34
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.termcat.business.areas;



import com.termcat.bd.beans.AreesFitxes;
import com.termcat.bd.exceptions.BaseRuntimeException;
import com.termcat.bd.exceptions.CreateException;
import com.termcat.bd.exceptions.FinderException;
import com.termcat.bd.exceptions.RemoveException;
import com.termcat.bd.utils.DBConnect;
import com.termcat.business.exceptions.AreaException;
import com.termcat.bd.beans.ArbreCamp;
import com.termcat.layout.commons.tree.DoubleClickTree;
import com.termcat.util.UIDGenerator;
import com.termcat.util.literales.Literales;
import electric.xml.Document;
import electric.xml.Element;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.List;
import java.util.Vector;
import com.termcat.layout.commons.lists.ItemListCustom;
import javax.swing.table.DefaultTableModel;

/**
 *
 * @author Vazquezl
 */
public class AreaManager {
    
    private static AreaManager manager;
    
    /** Creates a new instance of DictionaryManager */
    private AreaManager() {
    }
    
    public static AreaManager getManager(){
        if(manager==null){
            manager = new AreaManager();
        }
        return manager;
    }
    
    /*
     *       Mtode encarregat de donar d'alta una nova rea al sistema.
     *      @param idDiccionari:    Identificador del diccionari.
     *      @param idArea:          Identificador de l'area. En cas de ser null o buit el genera un generador
     *                              d'ids nics.
     *      @param idPare:          Identificador del pare
     *      @param nomArea:         Nom de l'rea.
     */
    public ArbreCamp altaArea(String idDiccionari,String idArea, String idPare, String nomArea) throws AreaException
    {
         try
         {
             ArbreCamp area=new ArbreCamp();
             if(idArea==null || idArea.equals(""))
                area.setIdArea(UIDGenerator.generateUniqueId());  
             else
                area.setIdArea(idArea);  
             area.setIdDiccionari(idDiccionari);
             area.setIdPare(idPare);
             area.setNomArea(nomArea);
             area.create(); 
             
             return area;
         }
         catch(Exception e)
         {
             e.printStackTrace();
             throw new AreaException(e.getMessage());
         }
    }
 
    /*
     *       Mtode encarregat de modificar els continguts d'una rea. Pot ser que s'indiquin tant el idPare i nomArea,
     *      com que noms s'especifiqui un d'ells.
     *      @param idArea:          Identificador de l'rea que es vol modificar.
     *      @param idPare:          Identificador del pare
     *      @param nomArea:         Nom de l'rea.
     *
     *       No el fa be a BD!
     */
    public ArbreCamp updateArea(String idArea, String idPare, String nomArea) throws AreaException
    {
         try
         {
             ArbreCamp newArea=new ArbreCamp();
             ArbreCamp area=(ArbreCamp)newArea.findByPrimaryKey(idArea);
             if (idPare!=null && !idPare.equals(""))
                area.setIdPare(idPare);
             if (nomArea!=null && !nomArea.equals(""))
                area.setNomArea(nomArea);
             
             area.flush();
             
             return area;
         }
         catch(Exception e)
         {
             e.printStackTrace();
             throw new AreaException(e.getMessage());
         }
    }
 
    /**
     *       Mtode que elimina els fills associats a un pare amb id determinat.
     *      @param idPare:    Identificador de l'rea del que es volen eliminar els fills.
     */
     /*public void deleteFills(String idPare) throws AreaException
     {
        try
        {                  
            ArbreCamp area=new ArbreCamp();
            area.removeChildrenOfParent(idPare);
        }
     
        catch (Exception e) 
        {
             e.printStackTrace();
             throw new AreaException(e.getMessage());
        }
     }
     */
      public void deleteFillsAssociacions(ArbreCamp ac) throws AreaException
     {
        try
        {                       
            // Cridem a la funci recursivament per a cadascn dels fills.
            List l=getChildren(ac.getIdDiccionari(),ac.getIdArea());           
            if(l.size()>0)
            {
                for (int i=0;i<l.size();++i)
                    deleteFillsAssociacions((ArbreCamp)l.get(i));                
            }
            // Esborrem el node.
            delete(ac.getIdArea());            
        }     
        catch (Exception e) 
        {
             e.printStackTrace();
             throw new AreaException(e.getMessage());
        }
     }
     
     /**
     *       Mtode que elimina una rea de la BD.
     *      @param idArea:    Identificador de l'rea que es vol eliminar.
     */
     public void delete(String idArea) throws AreaException
     {
        try
        {
            // Esborrem les associacions de fitxes amb aquesta area temtica.
            AreesFitxes af=new AreesFitxes();
            Vector<AreesFitxes> v=af.getAllAreesFitxesOfArea(idArea);
            for(int i=0;i<v.size();++i)            
            {
                System.out.println("Asso deleted:"+v.get(i).getIdArea());
                v.get(i).remove();
            }
            
            // Esborrem l'area temtica.
            ArbreCamp newArea=new ArbreCamp();
            ArbreCamp area =(ArbreCamp)newArea.findByPrimaryKey(idArea);
            area.remove();
        }
     
        catch (Exception e) 
        {
             e.printStackTrace();
             throw new AreaException(e.getMessage());
        }
     }
     
     /*
      *      Mtode que retorna el node arrel d'un diccionari concret.
      *     @param idDiccionari:    Identificador del diccionari.
      */
     public ArbreCamp getRootNodeTree(String idDiccionari) throws AreaException
     {
        ArbreCamp newArea=new ArbreCamp();
        ArbreCamp area=null;
        String sql="ID_DICCIONARI='"+idDiccionari+"' AND ID_AREA='root'";
        List list;
        
        // Tanquem la connexio per si un cas no s'ha tancat be.
        DBConnect.putConnectionTransactional();
        try 
        {
            list = (List) newArea.findByGenericWhere(sql);        
            if(list.size()==0)
                  area=altaArea(idDiccionari,"root","root",Literales.getTextForLiteral("Arees_Tematiques"));
            else
                  area = (ArbreCamp)list.get(0);
        }
         catch (FinderException ex) 
         {

            ex.printStackTrace();
        }
         return area;
     }
     
   /*
    *      Mtode que obte els fills d'un determinat pare.
    *     @param idDiccionari:     Identificador del diccionari.
    *     @param idPare:           Identificador del pare.
    */
     public List getChildren(String idDiccionari,String idPare)
     {
         List l=null;
         String sql=" ID_PARE='"+idPare+"' AND ID_DICCIONARI='"+idDiccionari+"' AND ID_PARE!=ID_AREA";
         try
         {
             ArbreCamp newArea= new ArbreCamp();        
             l=(List) newArea.findByGenericWhere(sql);
         }
         catch(Exception e)
         {
             // No fem res en aquest cas vol dir que l'objecte para no te cap fill associat.
         }

         return l;
     }
     
     public Vector getAreesOfFitxa(String idFitxa) throws AreaException
     {      
          Vector v=new Vector();
          ArbreCamp ac=new ArbreCamp();
          try 
          {
              v=ac.getAreesOfFitxa(idFitxa);
          }
          catch (FinderException e) 
          {
                e.printStackTrace();
                throw new AreaException(e.getMessage());   
          }
          return v;
     }
     
     
     public void altaAreaFitxa(String idArea, String idFitxa) throws AreaException
     {
         try
         {
             Connection con=DBConnect.getConnectionTransactional();
             AreesFitxes af=new AreesFitxes();
             af.setIdArea(idArea);
             af.setIdFitxa(idFitxa);
             af.create(con);         
         }
         catch (Exception e) 
         {
            e.printStackTrace();
            throw new AreaException(e.getMessage());   
        }
     }

    public void deleteAssociacio(String idAreaSelected, String idFitxa)  throws AreaException
    {
       Connection con=DBConnect.getConnectionTransactional();
        AreesFitxes newAf=new AreesFitxes();
        AreesFitxes af=new AreesFitxes();
        try 
        {
            af=(AreesFitxes)newAf.findByPrimaryKey(idAreaSelected,idFitxa);
            af.remove(con);
        }
        catch (Exception ex) 
        {
            ex.printStackTrace();
            throw new AreaException(ex.getMessage());            
        }        
    }

 
    public Element getAllArbreCampsToXML(String idDiccionari) throws AreaException
    {
         Document doc=new Document();
         Element elem=new Element("DadesArbreCamp");
         String sql=" ID_DICCIONARI='"+idDiccionari+"'";
         
         try
         {
            ArbreCamp newAc=new ArbreCamp();
            ArbreCamp ac=new ArbreCamp();

            List listAc=newAc.findByGenericWhere(sql);
            for(int i=0;i<listAc.size();++i)
            {
                ac = (ArbreCamp)listAc.get(i);
                elem.addElement(ac.toXml());
            }
            
            doc.setRoot(elem);

         }
         catch(Exception e)
         { 
            e.printStackTrace();
            throw new AreaException(e.getMessage());
         }

         return doc.getRoot();

    }

    public void addAllAreesFitxesOfDictionary(Element elem, String idDiccionari) 
    {
            Connection con = null;
            Statement stm = null;
            ResultSet rs = null;

            try 
            {
                   String sql="SELECT arees_fitxes.ID_AREA, arees_fitxes.ID_FITXA  " +
                       "FROM CO_AREES_FITXES arees_fitxes, CO_FITXES fitxes " +
                       "where fitxes.ID_FITXA = arees_fitxes.ID_FITXA and fitxes.ID_DICCIONARI='"+idDiccionari+"'";    
                   
                    con = DBConnect.getConnection();
                    stm = con.createStatement();
                    rs = stm.executeQuery(sql);
                    while (rs.next()) {
                        AreesFitxes item=new AreesFitxes();
                        item.setIdArea(rs.getString("ID_AREA"));
                        item.setIdFitxa(rs.getString("ID_FITXA"));
                        elem.addChild(item.toXml());
                    }
            }
            catch(Exception e)
            {
               throw new BaseRuntimeException(e.toString());                 
            }
            finally 
            {
                if (rs != null) try { rs.close(); } catch (Exception ignored) {}
                if (stm != null) try { stm.close(); } catch (Exception ignored) {}
                if (con != null) DBConnect.putConnection(con);
             }
     }        

    public List getAllArbreCamps(String idDiccionari) throws FinderException 
    {
        
        String sql=" ID_DICCIONARI='"+idDiccionari+"'";
        ArbreCamp ac=new ArbreCamp();               
        
        return ac.findByGenericWhere(sql);
    }          
   
    public void deleteAllArbreCamps() throws AreaException
    {
        String sql="delete from CO_ARBRE_CAMP";
        try 
        {
            com.termcat.bd.beans.BDUtil.execute(sql);        
        } 
        catch (Exception e) 
        {
            throw new AreaException(e.getMessage());
        }        
    }
    
    public void deleteAllAreesFitxes() throws AreaException
    {
        String sql="delete from CO_AREES_FITXES";
        try 
        {
            com.termcat.bd.beans.BDUtil.execute(sql);        
        } 
        catch (Exception e) 
        {
            throw new AreaException(e.getMessage());
        }        
    }
    
}
