/*
 * DictionaryManager.java
 *
 * Created on 13 de julio de 2007, 8:34
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.termcat.business.dictionary;


import com.termcat.bd.beans.Llengues;
import com.termcat.bd.beans.LlenguesDiccionari;
import com.termcat.bd.exceptions.FinderException;
import com.termcat.bd.utils.DBConnect;
import com.termcat.business.areas.AreaManager;
import com.termcat.business.exceptions.DictionaryException;
import com.termcat.bd.beans.Diccionaris;
import com.termcat.business.exceptions.DictionaryLanguageException;
import com.termcat.business.maintenance.font.FontManager;
import com.termcat.business.terminologic.contexts.ContextManager;
import com.termcat.business.terminologic.data.DataManager;
import com.termcat.business.terminologic.definitions.DefinitionManager;
import com.termcat.business.terminologic.denominations.DenominationManager;
import com.termcat.business.terminologic.notes.NoteManager;
import com.termcat.util.UIDGenerator;
import com.termcat.util.literales.Literales;
import electric.xml.Element;
import java.sql.SQLException;
import java.util.Vector;
import com.termcat.layout.commons.lists.ItemListCustom;

/**
 *
 * @author Vazquezl
 */
public class DictionaryManager {
    
    private static DictionaryManager manager;
    
    /** Creates a new instance of DictionaryManager */
    private DictionaryManager() {
    }
    
    public static DictionaryManager getManager(){
        if(manager==null){
            manager = new DictionaryManager();
        }
        return manager;
    }
    
    /*
     *       Mtode encarregat de donar d'alta un nou diccionari al sistema. L'identificador el genera un generador
     *      d'ids nics.
     *      @param nom:             Nom del diccionari.
     *      @param descripcio:      Descripcio del diccionari
     *      @param observacions:    Observacions del diccionari
     */
    public String altaDiccionari(String nom,String descripcio, String observacions) throws DictionaryException
    {
         try
         {
             Diccionaris diccionari=new Diccionaris();
             String id=UIDGenerator.generateUniqueId();
             diccionari.setIdDiccionari(id);  
             diccionari.setNom(nom);
             diccionari.setDescripcio(descripcio);         
             diccionari.setObservacions(observacions);
             diccionari.create(); 
             
             return id;
         }
         catch(Exception e)
         {
             e.printStackTrace();
             throw new DictionaryException(e.getMessage());
         }
    }
    
    // Funcio per a fer els imports.
    public void altaDiccionari(String idDiccionari,String nom,String descripcio, String observacions) throws DictionaryException
    {
         try
         {
             if (descripcio==null)
                 descripcio="";
             if (observacions==null)
                 observacions="";
             Diccionaris diccionari=new Diccionaris();
             diccionari.setIdDiccionari(idDiccionari);              
             diccionari.setNom(nom);
             diccionari.setDescripcio(descripcio);         
             diccionari.setObservacions(observacions);
             diccionari.create();             

         }
         catch(Exception e)
         {
             e.printStackTrace();
             throw new DictionaryException(e.getMessage());
         }
    }
    
    /*
     *       Mtode encarregat de modificar un diccionari existent al sistema.
     *      @param idDiccionari:    Identificador del diccionari que es vol modificar.
     *      @param nom:             Nom del diccionari.
     *      @param descripcio:      Descripcio del diccionari
     *      @param observacions:    Observacions del diccionari
     */
     public void updateDiccionari(String idDiccionari,String nom,String descripcio, String observacions) throws DictionaryException
    {
         try
         {
             Diccionaris newDiccionari=new Diccionaris();
             Diccionaris diccionari=(Diccionaris) newDiccionari.findByPrimaryKey(idDiccionari);
             
             diccionari.setNom(nom);
             diccionari.setDescripcio(descripcio);         
             diccionari.setObservacions(observacions);
             diccionari.flush(); 
             
         }
         catch(Exception e)
         {
             e.printStackTrace();
             throw new DictionaryException(e.getMessage());
         }
    }

    
    /*
     *       Mtode encarregat d'obtenir el nom d'un diccionari.
     *      @param idDiccionari:   Identificador del diccionari del que es vol saber el nom.
     *      @return nomDiccionari:  Nom del diccionari en qesti. 
     */
    public String getNomDiccionari(String idDiccionari) throws DictionaryException
    {
        String nomDiccionari=null;
        try
        {
            Diccionaris diccionari=new Diccionaris();
            Diccionaris diccSelected=(Diccionaris)diccionari.findByPrimaryKey(idDiccionari);
            nomDiccionari=diccSelected.getNom();
        }
        catch(Exception e)
        {
            // e.printStackTrace();
            //throw new DictionaryException(e.getMessage());            
            nomDiccionari="senseNom";
        }
        return nomDiccionari;
    }

  /*
   *    Mtode que troba les llengues associades a un diccionari en funci del id del diccionari.
   *  Retorna un Vector d'ids de llengues: [id1 | id2 | ... | idn]
   */
    public Vector getLlenguesOfDictionary(String idDiccionari) throws DictionaryException
    {
         try
        {
            LlenguesDiccionari llenguesDiccionari=new LlenguesDiccionari();
            return llenguesDiccionari.findLlenguesOfDictionary(idDiccionari);
            
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new DictionaryException(e.getMessage());            
        }
    }
  
    /**
     *       Mtode encarregat d'actualitzar les llengues i prioritats de les mateixes que pertanyen a un diccionari.
     *      @param idDiccionari:    Identificador del diccionari
     *      @param llengues:        Vector amb els ids de les llengues que es volen modificar/insertar pel diccionari indicat.      
     */
    public void setLlenguesOfDictionary(String idDiccionari,Vector llengues) throws DictionaryLanguageException
    {
        try
        {
          LlenguesDiccionari llenguesDiccionari=new LlenguesDiccionari();
          LlenguesDiccionari llenguesDiccionariSelected=new LlenguesDiccionari();
          String idLlengua=null;
          String itemId=null;
          ItemListCustom item=new ItemListCustom();
           for (int i=0;i<llengues.size();++i)
           {
                item=(ItemListCustom)llengues.get(i);
                itemId=item.getId();
                llenguesDiccionariSelected=existsLlenguaInDictionary(idDiccionari,itemId);

                //  Comprovem si la llengua ja existia, en aquest cas fem el canvi de la prioritat i guardem. En cas
                // que no existeixi creem una nova instancia de la classe.
                if(llenguesDiccionariSelected!=null)
                {
                     llenguesDiccionariSelected.setPrioritat((long)i);
                     llenguesDiccionariSelected.flush();
                }
                else
                {
                    llenguesDiccionari.setIdDiccionari(idDiccionari);
                    llenguesDiccionari.setIdLlengua(itemId);
                    llenguesDiccionari.setPrioritat((long)i);
                    llenguesDiccionari.create();
                }
           }
            
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new DictionaryLanguageException(e.getMessage());            
        }
    }
    
    /**
     *    Mtode encarregat de comprovar si existeix una llengua en un diccionari. En cas que no existeixi es provoca una
     *   excepci no es fa res al bloc catch i es retorna un null. --> S'hauria de comprovar que l'error que es provoca
     *   es el de PK not found.
     *
     *   @param idDiccionari:   Identificador del diccionari
     *   @param idLlengua:      Identificador de la llengua que es vol saber si existeix al diccionari indicat
     *
     *   @return LlenguesDiccionari:    si no existeix la llengua en el diccionari indicat--> null
     *                                  else --> l'objecte
     *            
     *   Una altra manera de fer-ho seria que en el catch de "setLlenguesOfDictionary" fer els sets i create en cas que la
     *   excepcio sigui per PK not found.                                  
     */
    private LlenguesDiccionari existsLlenguaInDictionary(String idDiccionari, String idLlengua) 
    {
        LlenguesDiccionari llengua=null;
        LlenguesDiccionari llenguaAux=new LlenguesDiccionari();
        try
        {
            llengua=(LlenguesDiccionari)llenguaAux.findByPrimaryKey(idDiccionari,idLlengua);
        }
        catch(Exception e)
        {
         // Faltaria fer una comprovaci de que en cas de que l'error NO sigui de PK fer un rethrow!   
        }
        return llengua;
    }

    /**
     *       Mtode que elimina les llengues deseleccionades.
     *      @param idDiccionari:    Identificador del diccionari del que es volen eliminar les llenges
     *      @param llengues:        Vector amb les llengues que es volen eliminar.
     */
     public void deleteLlenguesOfDictionary(String idDiccionari,Vector llengues) throws DictionaryLanguageException
     {
        LlenguesDiccionari llenguesDiccionariSelected=new LlenguesDiccionari();
        ItemListCustom item=new ItemListCustom();
        try 
        {
            for(int i=0;i<llengues.size();++i)
            {
                item=(ItemListCustom)llengues.get(i);
                llenguesDiccionariSelected=existsLlenguaInDictionary(idDiccionari,item.getId());

                // Noms eliminem la llengua en el cas en que existeixi per a aquell diccionari.
                if(llenguesDiccionariSelected!=null)
                    llenguesDiccionariSelected.remove();
            }    
        } 
        catch (Exception e) 
        {
            e.printStackTrace();
            throw new DictionaryLanguageException(e.getMessage());
        }
     }

    public String[] getDadesDiccionari(String idDiccionari) throws DictionaryException
    {
        String result[]= new String[3];
        try 
        {
            Diccionaris newDiccionari=new Diccionaris();
            Diccionaris diccionari=(Diccionaris) newDiccionari.findByPrimaryKey(idDiccionari);
            result[0]=diccionari.getNom();
            result[1]=diccionari.getDescripcio();
            result[2]=diccionari.getObservacions();          
        } 
        catch (FinderException e) 
        {
            e.printStackTrace();
            throw new DictionaryException(e.getMessage());
        }
        
        return result;        
    }
    
    //  Mtode que obte les dades del diccionari en format XML. S'utilitzar per a la exportaci de dades.
     public Element getDadesDiccionariToXML(String idDiccionari) throws DictionaryException
    {
        Element elem=new Element("DadesDiccionari");
        try 
        {
            Diccionaris newDiccionari=new Diccionaris();
            Diccionaris diccionari=(Diccionaris) newDiccionari.findByPrimaryKey(idDiccionari);
            elem=diccionari.toXml();
        } 
        catch (FinderException e) 
        {
            e.printStackTrace();
            throw new DictionaryException(e.getMessage());
        }
        
        return elem;
    }

    public String getDiccionariId() 
    {
        String id=null;
        Diccionaris newDiccionari=new Diccionaris();
        Diccionaris dic=null;
        try 
        {
            Vector allDic=newDiccionari.findAll();
            if (allDic!=null && allDic.size()>0)
            {
                dic=(Diccionaris) allDic.get(0);
                id=dic.getIdDiccionari();
            }
        } catch (FinderException ex) {}
        
        return id;        
    }

    public void deleteAllDiccionari() throws DictionaryException
    {
        String sql="delete from CO_DICCIONARIS";
        try 
        {
            com.termcat.bd.beans.BDUtil.execute(sql);        
        } 
        catch (Exception e) 
        {
            throw new DictionaryException(e.getMessage());
        }        
    }

    public void deleteAllLlenguesDiccionari() throws DictionaryException
    {
        String sql="delete from CO_LLENGUES_DICCIONARI";
        try 
        {
            com.termcat.bd.beans.BDUtil.execute(sql);        
        } 
        catch (Exception e) 
        {
            throw new DictionaryException(e.getMessage());
        }        
    }
    
    
    public  void deleteDiccionari() throws Exception
    {
    
       try{    
            java.sql.Connection ct=DBConnect.getConnectionTransactional(); 
        
            // Esborrem les 'fulles'
            AreaManager.getManager().deleteAllAreesFitxes();
            AreaManager.getManager().deleteAllArbreCamps();                        
                       
            FontManager.getManager().deleteAllFontsContextos();
            FontManager.getManager().deleteAllFontsDefinicions();
            FontManager.getManager().deleteAllFontsDenominacions();
            FontManager.getManager().deleteAllFontsNotes();
            
            // Esborrer els penultim nodes.
            DenominationManager.getManager().deleteAllDenominacions();            

            ContextManager.getManager().deleteAllContextes();
            DefinitionManager.getManager().deleteAllDefinitions();
            NoteManager.getManager().deleteAllNotes();
           
            deleteAllLlenguesDiccionari();           
            
            
            // Esborrem les fitxes
            DataManager.getManager().deleteAllFitxes();            
            
            // Esborrem el diccionari
            deleteAllDiccionari();
            
            DBConnect.getConnectionTransactional().commit();
            
       }
        catch (Exception e) 
        {
            e.printStackTrace();
            // Fem el rollback            
            try 
            {
                DBConnect.getConnectionTransactional().rollback();
                throw new DictionaryException(Literales.getTextForMessage("DeleteDiccionariKO"));
            }
            catch (SQLException ex) 
            {
                ex.printStackTrace();
                throw new DictionaryException(Literales.getTextForMessage("DeleteDiccionariKO"));  
                
                // En cas de no poder fer el rollback, s'hauria de esborrar tot el contingut de ls BD una altre vegada 
                // per a que no quedes inconsistent l'estat de la mateixa.
            }                       
        }
       finally{
           DBConnect.putConnectionTransactional();                
       }
       
            
    }
    
    
}
