/*
 * CategoryManager.java
 *
 * Created on 6 de agosto de 2007, 10:14
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.termcat.business.maintenance.category;

import com.termcat.bd.beans.skels.BdUtils;
import com.termcat.business.exceptions.CategoryException;
import com.termcat.bd.beans.Categories;
import com.termcat.util.UIDGenerator;
import java.util.Iterator;
import java.util.Vector;

import electric.xml.Document;
import electric.xml.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 * @author Mena
 */
public class CategoryManager {
    
    private static CategoryManager manager;
    
    private final int PRIORITAT_UP=1;
    private final int PRIORITAT_DOWN=-1;
    
    /** Creates a new instance of CategoryManager */
    public CategoryManager() {
    }
    
    public static CategoryManager getManager()
    {
        if(manager==null)
        {
            manager = new CategoryManager();
        }
        return manager;
    }
    
    public void altaCategoria(String idCategoria, String codi, String descripcio, long prioritat) throws CategoryException 
    {
        if (descripcio==null)
            descripcio="";
        try
        {
            Categories categoria=new Categories();
            categoria.setIdCategoria(idCategoria);  
            if (codi==null)
                codi="";
            categoria.setCodiCategoria(codi);
            categoria.setDescripcio(descripcio);
            categoria.setPrioritat(prioritat);
            categoria.create();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new CategoryException(e.getMessage());
        }        
    }
    
    /*
     *  Mtode que inserta una nova categoria a la BD.
     *  
     *  @param codi:         Codi de la categoria que s'inserta
     *  @param descripcion:  Descripci de la categoria que s'inserta.
     *  @param prioritat:    Prioritat de la categoria.
     */
    public void altaCategoria(String codi,String descripcio, long prioritat) throws CategoryException
    {       
        try
        {
            Categories categoria=new Categories();
            categoria.setIdCategoria(UIDGenerator.generateUniqueId());  
            categoria.setCodiCategoria(codi);
            categoria.setDescripcio(descripcio);
            categoria.setPrioritat(prioritat);
            categoria.create();
        }
        catch(Exception e)
        {
            throw new CategoryException(e.getMessage());
        }
    }
    
    /*
     *  Mtode que actualitza els valors d'una categoria de la BD.
     * 
     *  @param idCategoria:     Identificador de la categoria.
     *  @param codi:            Codi de la categoria.
     *  @param descripcio:      Descripci de la categoria. 
     */
    public void updateCategoria(String idCategoria,String codi,String descripcio) throws CategoryException
    {
        try
        {
            Categories categoria=new Categories();
            Categories categoriaSelected=(Categories)categoria.findByPrimaryKey(idCategoria);
            
            categoriaSelected.setCodiCategoria(codi);
            categoriaSelected.setDescripcio(descripcio);
            categoriaSelected.flush();
        }
        catch(Exception e)
        {
            throw new CategoryException(e.getMessage());
        }
    }

    /*
     *   Mtode que actualitza els valors d'una categoria de la BD. Es el mtode que s'utilitzar desde la 
     *  botonera d'ordre de l'aplicaci.
     * 
     *  @param idCategoria:     Identificador de la categoria.
     *  @param prioritat:       Index amb el que es modificar la prioritat. Els valors possibles son
     *                          -1: En cas de fer un 'down' de la categoria
     *                          1: En cas de fer un 'up' de la categoria
     */
    public void updatePrioritat(String idCategoria,long prioritat) throws CategoryException
    {
        try
        {
            if (prioritat==PRIORITAT_UP || prioritat==PRIORITAT_DOWN)
            {    
                Categories categoria=new Categories();
                Categories categoriaSelected=(Categories)categoria.findByPrimaryKey(idCategoria);
                categoriaSelected.setPrioritat(categoriaSelected.getPrioritat()+prioritat);
                categoriaSelected.flush();
           }
           else
               throw new CategoryException("Index de prioritat incorrecte.");
                
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new CategoryException(e.getMessage());
        }
    }

    /*
     *  Mtode que elimina una categoria de la BD.
     * 
     *  @param idFont:   Identificador de la categoria.
     */
    public void deleteCategoria(String idCategoria) throws CategoryException
    {
        try
        {
             Categories categoria=new Categories();
             Categories categoriaSelected=(Categories)categoria.findByPrimaryKey(idCategoria);
             categoriaSelected.remove();
        }
        catch (Exception e)
        {
            throw new CategoryException(e.getMessage());
        }
    }    

    /*
     *  Mtode que retorna totes les categories de la BD ordenades per prioritat
     */
    public Vector getAllCodeIdCategories() throws CategoryException
    {
        try
        {           
            Categories categoria=new Categories();
            Vector allCategories=(Vector) categoria.findAllCodeIdCategories();
            return allCategories;              
        }
        catch(Exception e)
        {
            throw new CategoryException(e.getMessage());
        }   
    }
    
    /*
     *  Mtode que retorna TOTES les categories disponibles a la BD ordenades ascendentment per prioritat.
     * 
     */
    public Vector getAllCategories() throws CategoryException
    {
        try
        {           
            Categories categoria=new Categories();
            Vector allCategories=(Vector) categoria.findAllOrderByPriority();
            return allCategories;              
        }
        catch(Exception e)
        {
            throw new CategoryException(e.getMessage());
        }   
    } 
    
    /*
     *  Mtode que retorna la segent prioritat. Es te en compte que les prioritats van desde 0 a n on 0 es la
     * categoria mes prioritria i n la menos prioritaria.
     *
     */
    public long getNextPrioritat() throws CategoryException
    {
        try
        {   
            long resultAsLong=-1;
            String result=null;

            String sql="select max(PRIORITAT) from CO_CATEGORIES";
            Element prioritatXML = BdUtils.executeQueryToXml(sql);

            NodeList rows=prioritatXML.getElementsByTagName("row");
            Node row = rows.item(0);
            NodeList rowChilds = row.getChildNodes();
            Node maxOrdre = rowChilds.item(0);
            result = maxOrdre.getAttributes().getNamedItem("value").getNodeValue();
            if (result!=null)
                resultAsLong=Long.parseLong(result);
            
            return resultAsLong+1;
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new CategoryException(e.getMessage());
        }
    }
    
    /*
     *  Mtode encarregat de refrescar TOTES les prioritats de la BD.
     */        
       public void updatePrioritats() throws CategoryException       
       {
           try
           {
               Categories categoria = new Categories();

               // Obtener de la BD las posiciones actuales ordenadas.
               String categoriaId;
               Vector allCategories=getAllCategories();

               // Modificamos las prioridades.
               for(int ind=0; ind<allCategories.size();ind++)
               {
                   categoria=(Categories)allCategories.get(ind);
                   categoriaId=categoria.getIdCategoria();
                   // Hacemos que la regla por defecto siempre tenga valor -1.
                   setPrioritat(categoriaId,(long)ind);
                }        
           }
           catch(Exception e)
           {
                e.printStackTrace();
                throw new CategoryException(e.getMessage());               
           }
       }
    
    /*
     *   Mtode encarregat d'assignar una prioritat a una categoria amb un idDeterminat.
     *
     *  @param idCategoria:  Identificador de la categoria que a la que se li vol assignar una prioritat.
     *  @param prioritat:    Prioritat que es vol assignar.
     *
     */
     private void setPrioritat(String idCategoria,long prioritat) throws CategoryException
    {
        try
        {
            Categories categoria=new Categories();
            Categories categoriaSelected=(Categories)categoria.findByPrimaryKey(idCategoria);
            categoriaSelected.setPrioritat(prioritat);
            categoriaSelected.flush();                
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new CategoryException(e.getMessage());
        }
    }
     
    /*
     *  Mtode que obte el codi de la categoria que te per idCategoria=@id
     * 
     */
    public String getCodiCategoria(String id)throws CategoryException
    {
        String codiCategoria=null;
        try
        {
            Categories categoria=new Categories();
            Categories categoriaSelected=(Categories)categoria.findByPrimaryKey(id);
            codiCategoria=categoriaSelected.getCodiCategoria();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new CategoryException(e.getMessage());
        }
        return codiCategoria;
    }

    public void addAllCategoriesToXML(Element elem) throws CategoryException
    {
        try
        {           
            Categories categoria=new Categories();
            categoria.addAllToXml(elem);
        }
        catch(Exception e)
        {
            throw new CategoryException(e.getMessage());
        }   
    } 

    public void deleteAllCategories() throws CategoryException
    {
        String sql="delete from CO_CATEGORIES";
        try 
        {
            com.termcat.bd.beans.BDUtil.execute(sql);        
        } 
        catch (Exception e) 
        {
            throw new CategoryException(e.getMessage());
        }        
    }    
}
