/*
 * CategoryManager.java
 *
 * Created on 6 de agosto de 2007, 10:14
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.termcat.business.maintenance.hyerarchy;

import com.termcat.bd.beans.skels.BdUtils;
import com.termcat.business.exceptions.HyerarchyException;
import com.termcat.bd.beans.Jerarquies;
import com.termcat.util.UIDGenerator;
import java.util.Iterator;
import java.util.Vector;

import electric.xml.Document;
import electric.xml.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 * @author Mena
 */
public class HyerarchyManager {
    
    private static HyerarchyManager manager;
    
    private final int PRIORITAT_UP=1;
    private final int PRIORITAT_DOWN=-1;
    
    /** Creates a new instance of CategoryManager */
    public HyerarchyManager() {
    }
    
    public static HyerarchyManager getManager()
    {
        if(manager==null)
        {
            manager = new HyerarchyManager();
        }
        return manager;
    }

    public void altaJerarquia(String idJerarquia, String codi, String descripcio, long prioritat) throws HyerarchyException 
    {
       try
        {
           if(descripcio==null)
               descripcio="";

           // El cas de idJerarquia==null ja s'inserta al generar la BD mitjanant l'script.
           if (idJerarquia!=null && codi!=null)
           {
                Jerarquies jerarquia=new Jerarquies();
                jerarquia.setIdJerarquia(idJerarquia);
                jerarquia.setCodiJerarquia(codi);
                jerarquia.setDescripcio(descripcio);
                jerarquia.setPrioritat(prioritat);
                jerarquia.create();
           }
        }
        catch(Exception e)
        {
            throw new HyerarchyException(e.getMessage());
        }        
    }

    /*
     *  Mtode que inserta una nova font a la BD.
     *  
     *  @param codi:         Codi de la jerarquia que s'inserta
     *  @param descripcion:  Descripci de la jerarquia que s'inserta.
     *  @param prioritat:    Prioritat de la jerarquia.
     */
    public void altaJerarquia(String codi,String descripcio, long prioritat) throws HyerarchyException
    {       
        try
        {
            Jerarquies jerarquia=new Jerarquies();
            jerarquia.setIdJerarquia(UIDGenerator.generateUniqueId());  
            jerarquia.setCodiJerarquia(codi);
            jerarquia.setDescripcio(descripcio);
            jerarquia.setPrioritat(prioritat);
            jerarquia.create();
        }
        catch(Exception e)
        {
            throw new HyerarchyException(e.getMessage());
        }
    }
    
    /*
     *  Mtode que actualitza els valors d'una jerarquia de la BD.
     * 
     *  @param idCategoria:     Identificador de la jerarquia.
     *  @param codi:            Codi de la jerarquia.
     *  @param descripcio:      Descripci de la jerarquia. 
     */
    public void updateJerarquia(String idJerarquia,String codi,String descripcio) throws HyerarchyException
    {
        try
        {
            Jerarquies jerarquia=new Jerarquies();
            Jerarquies jerarquiaSelected=(Jerarquies)jerarquia.findByPrimaryKey(idJerarquia);
            
            jerarquiaSelected.setCodiJerarquia(codi);
            jerarquiaSelected.setDescripcio(descripcio);
            jerarquiaSelected.flush();
        }
        catch(Exception e)
        {
            throw new HyerarchyException("[Update jerarquia] "+e.getMessage());
        }
    }

    /*
     *   Mtode que actualitza els valors d'una jerarquia de la BD. Es el mtode que s'utilitzar desde la 
     *  botonera d'ordre de l'aplicaci.
     * 
     *  @param idCategoria:     Identificador de la jerarquia.
     *  @param prioritat:       Index amb el que es modificar la prioritat. Els valors possibles son
     *                          -1: En cas de fer un 'down' de la jerarquia
     *                          1: En cas de fer un 'up' de la jerarquia
     */
    public void updatePrioritat(String idJerarquia,long prioritat) throws HyerarchyException
    {
        try
        {
            if (prioritat==PRIORITAT_UP || prioritat==PRIORITAT_DOWN)
            {    
                Jerarquies jerarquia=new Jerarquies();
                Jerarquies jerarquiaSelected=(Jerarquies)jerarquia.findByPrimaryKey(idJerarquia);
                jerarquiaSelected.setPrioritat(jerarquiaSelected.getPrioritat()+prioritat);
                jerarquiaSelected.flush();
           }
           else
               throw new HyerarchyException("Index de prioritat incorrecte.");
                
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new HyerarchyException(e.getMessage());
        }
    }

    /*
     *  Mtode que elimina una jerarquia de la BD.
     * 
     *  @param idFont:   Identificador de la jerarquia.
     */
    public void deleteCategoria(String idJerarquia) throws HyerarchyException
    {
        try
        {
             Jerarquies jerarquia=new Jerarquies();
             Jerarquies jerarquiaSelected=(Jerarquies)jerarquia.findByPrimaryKey(idJerarquia);
             jerarquiaSelected.remove();
        }
        catch (Exception e)
        {
            throw new HyerarchyException(e.getMessage());
        }
    }    

    /*
     *  Mtode que obte TOTES les categories disponibles a la BD ordenades ascendentment per prioritat.
     * 
     */
    public Vector getAllJerarquies() throws HyerarchyException
    {
        try
        {           
            Jerarquies jerarquia=new Jerarquies();
            Vector allJerarquies=(Vector) jerarquia.findAllOrderByPriority();
            return allJerarquies;              
        }
        catch(Exception e)
        {
            throw new HyerarchyException(e.getMessage());
        }   
    } 
    /*
     *  Mtode que obte TOTES les categories disponibles a la BD ordenades ascendentment per prioritat.
     * 
     */
    public Vector getAllCodeIdJerarquies() throws HyerarchyException
    {
        try
        {           
            Jerarquies jerarquia=new Jerarquies();
            Vector allJerarquies=(Vector) jerarquia.findAllCodeIdJerarquies();
            return allJerarquies;              
        }
        catch(Exception e)
        {
            throw new HyerarchyException(e.getMessage());
        }   
    } 
    
    /*
     *  Mtode que retorna la segent prioritat. Es te en compte que les prioritats van desde 0 a n on 0 es la
     * jerarquia mes prioritria i n la menos prioritaria.
     *
     */
    public long getNextPrioritat() throws HyerarchyException
    {
        try
        {   
            long resultAsLong=-1;
            String result=null;

            String sql="select max(PRIORITAT) from CO_JERARQUIES";
            Element prioritatXML = BdUtils.executeQueryToXml(sql);

            NodeList rows=prioritatXML.getElementsByTagName("row");
            Node row = rows.item(0);
            NodeList rowChilds = row.getChildNodes();
            Node maxOrdre = rowChilds.item(0);
            result = maxOrdre.getAttributes().getNamedItem("value").getNodeValue();
            if (result!=null)
                resultAsLong=Long.parseLong(result);
            
            return resultAsLong+1;
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new HyerarchyException(e.getMessage());
        }
    }
    /*
     *  Mtode encarregat de refrescar TOTES les prioritats de la BD.
     *
     */        
       public void updatePrioritats() throws HyerarchyException
       {
           Jerarquies jerarquia = new Jerarquies();

           // Obtener de la BD las posiciones actuales ordenadas.
           String jerarquiaId;
           Vector allJerarquies=getAllJerarquies();

           // Modificamos las prioridades.
           int pos=0;
           for(int ind=0; ind<allJerarquies.size();ind++)
           {
               jerarquia=(Jerarquies)allJerarquies.get(ind);
               jerarquiaId=jerarquia.getIdJerarquia();
               if(!jerarquiaId.trim().equals("")){ //no se tenen en cuenta la jerarqui dummy con identificador vacio
               // Hacemos que la regla por defecto siempre tenga valor -1.
                setPrioritat(jerarquiaId,(long)pos);
                pos++;
               } 
            }        
       }
    
    /*
     *   Mtode encarregat d'assignar una prioritat a una jerarquia amb un idDeterminat.
     *
     *  @param idCategoria:  Identificador de la jerarquia que a la que se li vol assignar una prioritat.
     *  @param prioritat:    Prioritat que es vol assignar.
     *
     */
     private void setPrioritat(String idJerarquia,long prioritat) throws HyerarchyException
    {
        try
        {
            Jerarquies jerarquia=new Jerarquies();
            Jerarquies jerarquiaSelected=(Jerarquies)jerarquia.findByPrimaryKey(idJerarquia);
            jerarquiaSelected.setPrioritat(prioritat);
            jerarquiaSelected.flush();                
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new HyerarchyException(e.getMessage());
        }
    }
     
    /*
     *  Mtode que obte el codi de la jerarquia que te per idJerarquia=@id
     *  Pot ser null.
     */
    public String getCodiJerarquia(String id) throws HyerarchyException
    {
        String codiJerarquia=null;
        try
        {
            Jerarquies jerarquia=new Jerarquies();
            Jerarquies jerarquiaSelected=(Jerarquies)jerarquia.findByPrimaryKey(id);
            codiJerarquia=jerarquiaSelected.getCodiJerarquia();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            //throw new HyerarchyException(e.getMessage());
        }
        return codiJerarquia;
    }

    public void addAllJerarquiesToXML(Element elem) throws HyerarchyException
    {
        try
        {           
            Jerarquies jerarquia=new Jerarquies();
            jerarquia.addAllToXml(elem);
        }
        catch(Exception e)
        {
            throw new HyerarchyException(e.getMessage());
        }   
    }

    public void deleteAllJerarquies() throws HyerarchyException
    {
        String sql="delete from CO_JERARQUIES";
        try 
        {
            com.termcat.bd.beans.BDUtil.execute(sql);        
        } 
        catch (Exception e) 
        {
            throw new HyerarchyException(e.getMessage());
        }        
    }     
    
}
