/*
 * LanguageManager.java
 *
 * Created on 18 de julio de 2007, 18:16
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.termcat.business.maintenance.language;

import com.termcat.bd.beans.LlenguesDiccionari;
import com.termcat.bd.exceptions.CreateException;
import com.termcat.business.exceptions.LanguageException;
import com.termcat.bd.beans.Llengues;
import com.termcat.util.UIDGenerator;
import electric.xml.Document;
import electric.xml.Element;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
/**
 *
 * @author Mena
 */
public class LanguageManager 
{
    
   private static LanguageManager manager;
    
    /** Creates a new instance of DictionaryManager */
    private LanguageManager() {}
    
    public static LanguageManager getManager()
    {
        if(manager==null)
        {
            manager = new LanguageManager();
        }
        return manager;
    }
    
    /*
     *  Mtode que inserta una nova llengua a la BD.
     *  
     *  @param nombre:      Nom de la llengua que s'inserta
     *  @param descripcion: Descripci de la llengua que s'inserta.
     *
     */
    public void altaLlengua(String nom,String descripcio) throws LanguageException
    {       
        try
        {
            Llengues llengua=new Llengues();
            llengua.setCodiLlengua(nom);     
            llengua.setIdLlengua(UIDGenerator.generateUniqueId());      
            llengua.setNom(descripcio);         
            llengua.create();           
            System.out.println("Llengua donada d'alta amb parmetres "+nom+ ", "+descripcio);
        }
        catch(Exception e)
        {
            throw new LanguageException(e.getMessage());
        }
    }
    
    // Metode per l'import.
    public void altaLlengua(String idLlengua, String codiLlengua, String nom) throws LanguageException 
    {
        try
        {
            if (nom==null)
                nom="";
            Llengues llengua=new Llengues();
            llengua.setCodiLlengua(codiLlengua);     
            llengua.setIdLlengua(idLlengua);      
            llengua.setNom(nom);         
            llengua.create();           
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new LanguageException(e.getMessage());
        }        
    }    
    
    /*
     *  Mtode que actualitza els valors d'una llengua de la BD.
     * 
     *  @param idLlengua:       Identificador de la llengua.
     *  @param nombre:          Nom de la llengua.
     *  @param descripcio:      Descripci de la llengua. 
     */
    public void updateLlengua(String idLlengua,String nom,String descripcio) throws LanguageException
    {
        try
        {
            Llengues llengua=new Llengues();
            Llengues llenguaSelected=(Llengues)llengua.findByPrimaryKey(idLlengua);

            llenguaSelected.setCodiLlengua(nom);
            llenguaSelected.setNom(descripcio);
            llenguaSelected.flush();
        }
        catch(Exception e)
        {
            throw new LanguageException(e.getMessage());
        }
    }
    
    /*
     *  Mtode que elimina una llengua de la BD.
     * 
     *  @param idLlengua:   Identificador de la llengua.
     */
    public void deleteLlengua(String idLlengua) throws LanguageException
    {
        try
        {
            Llengues llengua=new Llengues();
            Llengues llenguaSelected=(Llengues)llengua.findByPrimaryKey(idLlengua);
            llenguaSelected.remove();
        }
        catch (Exception e)
        {
            throw new LanguageException(e.getMessage());
        }
    }
        
    /*
     *  Mtode que obte TOTES les llengues disponibles a la BD.
     * 
     */
    public Vector getAllLlengues() throws LanguageException
    {
        try
        {           
            Llengues llengua=new Llengues();
            Vector allLlengues=llengua.findAll();

            return allLlengues;              
        }
        catch(Exception e)
        {
            throw new LanguageException(e.getMessage());
        }   
    }  
    
       /*
     *  Mtode que obte TOTES les llengues disponibles a la BD.
     * 
     */
    public Vector getAllCodeIdLlengues() throws LanguageException
    {
        try
        {           
            Llengues llengua=new Llengues();
            Vector allLlengues=llengua.findAllCodeIdLlengues();

            return allLlengues;              
        }
        catch(Exception e)
        {
            throw new LanguageException(e.getMessage());
        }   
    } 
    
     /*
     *  Mtode que obte TOTES les llengues en funcio del diccionari
     * 
     */
    public Vector getLlenguesOfDictionary(String id) throws LanguageException
    {
        try
        {           
            LlenguesDiccionari llengua=new LlenguesDiccionari();
            Vector allLlengues=llengua.findLlenguesOfDictionary(id);
            return allLlengues;              
        }
        catch(Exception e)
        {
            throw new LanguageException(e.getMessage());
        }   
    } 
    
    /*
     *  Mtode que obte el codi de la llengua que te per idLlengua=@id
     * 
     */
    public String getCodiLlengua(String id) throws LanguageException
    {
        String codiLlengua=null;
        try
        {
            Llengues llengua=new Llengues();
            Llengues llenguaSelected=(Llengues)llengua.findByPrimaryKey(id);
            codiLlengua=llenguaSelected.getCodiLlengua();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new LanguageException(e.getMessage());            
        }
        return codiLlengua;
    }
    
    public void addAllLlenguesOfDictionaryToXML(String idDiccionari, Element elem) throws LanguageException
    {
         //Document doc=new Document();
         //Element elem=new Element("DadesLlengues");
         String sql=" ID_DICCIONARI='"+idDiccionari+"'";
         
         try
         {
            LlenguesDiccionari newLlenguesDiccionari=new LlenguesDiccionari();
            LlenguesDiccionari llenguesDiccionari=new LlenguesDiccionari();

            List list=newLlenguesDiccionari.findByGenericWhere(sql);
            for(int i=0;i<list.size();++i)
            {
                llenguesDiccionari = (LlenguesDiccionari)list.get(i);
                elem.addElement(llenguesDiccionari.toXml());
            }
            
            //doc.setRoot(elem);

         }
         catch(Exception e)
         {
             throw new LanguageException(e.getMessage());
         }

         //return doc.getRoot();

    }
      
    public void addAllLlenguesToXML(Element elem) throws LanguageException
    {
        try
        {           
            Llengues llengua=new Llengues();            
            llengua.findAllXml(elem);                           
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new LanguageException(e.getMessage());

        }   
    }  

    // Mtode per l'import'
    public void altaLlenguesDiccionari(String idDiccionari, String idLlengua, long prioritat) throws LanguageException 
    {
        LlenguesDiccionari llenguesDiccionari=new LlenguesDiccionari();   
        llenguesDiccionari.setIdDiccionari(idDiccionari);
        llenguesDiccionari.setIdLlengua(idLlengua);
        llenguesDiccionari.setPrioritat(prioritat);
        try
        {        
            llenguesDiccionari.create();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new LanguageException(e.getMessage());            
        }
    }   

    public void deleteAllLlengues() throws LanguageException
    {
        String sql="delete from CO_LLENGUES";
        try 
        {
            com.termcat.bd.beans.BDUtil.execute(sql);        
        } 
        catch (Exception e) 
        {
            throw new LanguageException(e.getMessage());
        }        
    }  
    
}   