/*
 * TermcatComparator.java
 *
 * Created on 12 de noviembre de 2007, 17:17
 *
 */

package com.termcat.business.reports.llistat.sortable;

import com.termcat.business.reports.llistat.LlistatReportManager;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

/**
 *
 * @author Vazquezl
 */
public class TermcatComparator implements Comparator{
    
    
    
    public TermcatComparator() {
    }
    
    
    
    public int compare(Object obj1, Object obj2) {
        int result = 0;
        
        try{        
            ComparableTermcat terme1 = (ComparableTermcat)obj1;
            ComparableTermcat terme2 = (ComparableTermcat)obj2;
            String termeRtf1 = terme1.getTermeRtf();
            String termeRtf2 = terme2.getTermeRtf();
            if(termeRtf1==null ){ //un nulo se cnsidera mas pequeo
                if(termeRtf2!=null)result = -1;
            }
            else if(termeRtf2==null) {
                result = 1;
            }
            else if(!termeRtf1.equals(termeRtf2)){
               result =  compareFirstLevel(terme1,terme2);
               if(result==0){ //no se han encontrado diferencias en el primer nivel pasamos al segundo
                   result = compareSecondLevel(terme1,terme2);
               }
               if(result==0){ //no se han encontrado diferencias en el segundo nivel pasamos al tercero
                   result = compareThirdLevel(terme1,terme2);
               }
               if(result==0){ //no se han encontrado diferencias en el tercer nivel pasamos al cuarto
                   result = compareFourthLevel(terme1,terme2);
               }
               if(result==0){ //no se han encontrado diferencias en el cuarto nivel delegamos la camparacion en java
                   result = terme1.getTermeNormal().compareTo(terme2.getTermeNormal());
               }
               
            }
        }
        catch(Throwable th){
            System.out.println("Error producido durante la comparacion!!");
            th.printStackTrace();
        }
        return result;
    }
    
    /**
     * Primer nivel : cifras y letras sin tener en cuenta las mayusculas
     * numeros : estaran precedidos y seguidos de un caracter en blanco. La marca del millar no se tiene en cuenta y la coma
     *           sera el indicador decimal
     *           las cifas romanas no se tratan
     *           los acentos no se tienen en cuenta ni tampoco los caracteres no alfanumericos
     **/
    private int compareFirstLevel(ComparableTermcat terme1,ComparableTermcat terme2){
        int result = 0;
        int numWords1 = 0;
        int numWords2 = 0;
        String termeNormal1 = terme1.getTermeNormal().toLowerCase(LlistatReportManager.LOCALE);
        String termeNormal2 = terme2.getTermeNormal().toLowerCase(LlistatReportManager.LOCALE);
        //eliminamos los caracteres fuera de rango que para este nivel no aplican
        termeNormal1 = SortUtils.simplifyString(termeNormal1,SortUtils.FIRST_LEVEL);
        termeNormal2 = SortUtils.simplifyString(termeNormal2,SortUtils.FIRST_LEVEL);
        
        if(!termeNormal1.equals(termeNormal2)){ //si son iguales sin tener en cuenta las mayusculas y acentos este nivel retorna iguales
            if(termeNormal1.trim().equals("")&&!termeNormal2.trim().equals("")){ //el resultado de simplificar da vacio -> el caracter eliminado es especial y por lo tanto la palabra es menos prioritaria
                result = 1;
            }
            else if(!termeNormal1.trim().equals("")&&termeNormal2.trim().equals("")){
                result = -1;
            }
            else if (termeNormal1.trim().equals("")&&termeNormal2.trim().equals("")){
                result = 0;
            }
            else{ //ninguno de los dos palabras es vacio
                String [] wordsTerme1 = SortUtils.splitString(termeNormal1);
                String [] wordsTerme2 = SortUtils.splitString(termeNormal2);
                numWords1 = wordsTerme1.length;
                numWords2 = wordsTerme2.length;
                int minNumWords = Math.min(numWords1,numWords2);
                for (int i=0;i<minNumWords;i++){
                    result = SortUtils.compareLikeNumber(wordsTerme1[i],wordsTerme2[i]);
                    if (result!=0) break;
                    else{ //comparar palabras
                       result= SortUtils.compareLikeWord(wordsTerme1[i],wordsTerme2[i]);
                       if (result!=0) break;
                    }
                }
            }
        }
        if(result==0){ //si despues de comparar no hay diferencias el mayor sera aquel que tenga mas palabras si no lo hubiese serian iguales
            if(numWords1>numWords2) result = 1;
            else if(numWords1<numWords2) result = -1;
        }
        return result;
    }
   
    /**
     * En este nivel se tienen en cuenta los acentos. Solo se llega a este nivel si en la comparacion de nive1 no se
     *han encontrado diferencias
     **/
    private int compareSecondLevel(ComparableTermcat terme1,ComparableTermcat terme2){
        int result = 0;
        int numWords1 = 0; 
        int numWords2 = 0;
        
        String termeNormal1 = terme1.getTermeNormal().toLowerCase(LlistatReportManager.LOCALE);
        String termeNormal2 = terme2.getTermeNormal().toLowerCase(LlistatReportManager.LOCALE);
        //eliminamos los caracteres fuera de rango que para este nivel no aplican
        termeNormal1 = SortUtils.simplifyString(termeNormal1,SortUtils.SECOND_LEVEL);
        termeNormal2 = SortUtils.simplifyString(termeNormal2,SortUtils.SECOND_LEVEL);
        if(!termeNormal1.equals(termeNormal2)){ //no son iguales teniendo en cuenta los signos de puntuacion -> hay que buscar las diferencias
            
            String [] wordsTerme1 = SortUtils.splitString(termeNormal1);
            String [] wordsTerme2 = SortUtils.splitString(termeNormal2);
            numWords1 = wordsTerme1.length;
            numWords2 = wordsTerme2.length;
            int minNumWords = Math.min(numWords1,numWords2);
            for (int i=0;i<minNumWords;i++){
                result = SortUtils.compareLikeAccentuatedWord(wordsTerme1[i],wordsTerme2[i]);
                if (result!=0) break;
            }
            
        }
        if(result==0){ //si despues de comparar no hay diferencias el mayor sera aquel que tenga mas palabras si no lo hubiese serian iguales
            if(numWords1>numWords2) result = 1;
            else if(numWords1<numWords2) result = -1;
        }
        
        return result;
    }
    
    /**
     *En este nivel se tienen en cuenta las mayusculas
     **/
    private int compareThirdLevel(ComparableTermcat terme1,ComparableTermcat terme2){
        int result = 0;
        int numWords1 = 0;
        int numWords2 = 0;
        
        String termeNormal1 = terme1.getTermeNormal();
        String termeNormal2 = terme2.getTermeNormal();
        //eliminamos los caracteres fuera de rango que para este nivel no aplican
        termeNormal1 = SortUtils.simplifyString(termeNormal1,SortUtils.THIRD_LEVEL);
        termeNormal2 = SortUtils.simplifyString(termeNormal2,SortUtils.THIRD_LEVEL);
        if(!termeNormal1.equals(termeNormal2)){ //no son iguales hay alguna letra mayuscula / minuscula que difiere
            String [] wordsTerme1 = SortUtils.splitString(termeNormal1);
            String [] wordsTerme2 = SortUtils.splitString(termeNormal2);
            numWords1 = wordsTerme1.length;
            numWords2 = wordsTerme2.length;
            int minNumWords = Math.min(numWords1,numWords2);
            for (int i=0;i<minNumWords;i++){  
                result = SortUtils.compareLikeKeySensitiveWord(wordsTerme1[i],wordsTerme2[i]);
                if (result!=0) break;
            }
        }
        if(result==0){ //si despues de comparar no hay diferencias el mayor sera aquel que tenga mas palabras si no lo hubiese serian iguales
            if(numWords1>numWords2) result = 1;
            else if(numWords1<numWords2) result = -1;
        }
        
        return result;
    }

    /**
     *En este nivel se tienen en cuenta los caracteres especiales
     **/
    private int compareFourthLevel(ComparableTermcat terme1,ComparableTermcat terme2){
        int result = 0;
        String termeNormal1 = terme1.getTermeNormal();
        String termeNormal2 = terme2.getTermeNormal();
        int numWords1 = 0;
        int numWords2 = 0;

        if(!termeNormal1.equals(termeNormal2)){ //no son iguales hay alguna letra mayuscula / minuscula que difiere
            String [] wordsTerme1 = SortUtils.splitString(termeNormal1);
            String [] wordsTerme2 = SortUtils.splitString(termeNormal2);
            numWords1 = wordsTerme1.length;
            numWords2 = wordsTerme2.length;
            int minNumWords = Math.min(numWords1,numWords2);
            for (int i=0;i<minNumWords;i++){  
                result = SortUtils.compareLikeSpecialWord(wordsTerme1[i],wordsTerme2[i]);
                if (result!=0) break;
            }
        }
        if(result==0){ //si despues de comparar no hay diferencias el mayor sera aquel que tenga mas palabras si no lo hubiese serian iguales
            if(numWords1>numWords2) result = 1;
            else if(numWords1<numWords2) result = -1;
        }
        return result;
    }
    
    
    public static void main (String [] args){
        /*StringBuffer stf = new StringBuffer("1".toUpperCase(LlistatReportManager.LOCALE));
        StringBuffer stf2 = new StringBuffer("c".toUpperCase(LlistatReportManager.LOCALE));
        StringBuffer stf3 = new StringBuffer("");
        System.out.println(stf.codePointAt(0));
        System.out.println(stf2.codePointAt(0));
        System.out.println(stf3.codePointAt(0));
        java.util.Vector v = new java.util.Vector();
        v.add(new Float("111.1"));
        v.add(new Float("111.2"));
        v.add(new Float("10"));
        v.add(new Float("11"));
        Collections.sort(v);
        System.out.println(v);*/
        
        try{
          BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
          String linea = null;
          Vector <ComparableTermcat> list = new Vector<ComparableTermcat>();
          while (!" ".equals(linea)){
              linea = br.readLine(); 
              com.termcat.business.reports.llistat.BeanFichaData data = new com.termcat.business.reports.llistat.BeanFichaData();
              data.setDenominacio(linea);
              data.setTermeRtf(linea);
              list.add(data);
           }  
          System.out.println("Resultado de ordenar..");
          Collections.sort(list,new TermcatComparator());
          for(int i=0;i<list.size();i++){
            System.out.println(list.get(i).getTermeNormal());
          }
        }
        catch(Exception e){
            e.printStackTrace();
        }
        
    }
}
