/*
 * DataManager.java
 *
 * Created on 7 de agosto de 2007, 10:04
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.termcat.business.terminologic.data;

import com.termcat.bd.beans.Denominacions;
import com.termcat.bd.beans.Fitxes;
import com.termcat.bd.exceptions.FinderException;
import com.termcat.bd.utils.DBConnect;
import com.termcat.business.exceptions.DataException;
import com.termcat.layout.commons.BoolString;
import com.termcat.layout.commons.lists.ItemListCustom;
import com.termcat.layout.commons.search.Search;
import com.termcat.layout.commons.search.Values;
import com.termcat.util.UIDGenerator;
import electric.xml.Document;
import electric.xml.Element;
import java.sql.Connection;
import java.util.Vector;
import java.util.List;

/**
 *
 * @author Mena
 */
public class DataManager {
        
    private static DataManager manager;

    /** Creates a new instance of DataManager */
    private DataManager() {
    }
    
    public static DataManager getManager()
    {
        if(manager==null)
        {
            manager = new DataManager();
        }
     
       return manager;
    }
        
     
    /*
     *  Mtode que inserta una nova fitxa a la BD.
     *  
     */
    public String altaFitxa(String idDiccionari) throws DataException
    {       
        try
        {
            Connection con=DBConnect.getConnectionTransactional();
            Fitxes fitxa=new Fitxes();
            String idFitxa=UIDGenerator.generateUniqueId();
            fitxa.setIdFitxa(idFitxa);     
            fitxa.setIdDiccionari(idDiccionari);
            fitxa.create(con);           
            return idFitxa;
        }
        catch(Exception e)
        {
            throw new DataException(e.getMessage());
        }
    }

    // Funcio per als imports.
    public void altaFitxa(String idFitxa, String idDiccionari,String observacions, String imagePath) throws DataException
    {       
        try
        {
            Connection con=DBConnect.getConnectionTransactional();
            Fitxes fitxa=new Fitxes();
            fitxa.setIdFitxa(idFitxa);     
            fitxa.setIdDiccionari(idDiccionari);
            if (observacions==null)
                observacions="";
            fitxa.setObservacions(observacions);
            fitxa.setImagePath("");           
            fitxa.create(con);           

        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new DataException(e.getMessage());
            
        }
    }
    /*
     *  Mtode que elimina una fitxa de la BD.
     * 
     *  @param idFont:   Identificador de la font.
     */
    public void deleteFitxa(String idFitxa) throws DataException
    {
        try
        {
            //Connection con=DBConnect.getConnectionTransactional();
            Fitxes fitxa=new Fitxes();
            Fitxes fitxaSelected=(Fitxes)fitxa.findByPrimaryKey(idFitxa);
            fitxaSelected.remove();
            //fitxaSelected.remove(con);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            
           throw new DataException(e.getMessage());
        }
    }
   
    /*
     *       Mtode que retorna totes les fitxes disponibles a un determinat diccionari.
     *      @param idDiccionari:    Identificador del diccionari.
     */
     public List getAllFitxes(String idDiccionari) throws DataException
     {
         try
         {
            Fitxes fitxa=new Fitxes();
            String sql=" ID_DICCIONARI='"+idDiccionari+"'";
            return fitxa.findByGenericWhere(sql);
         }
         catch(Exception e)
         {
             e.printStackTrace();
             throw new DataException(e.getMessage());
         }       
    }     
    
   /*
    *        Mtode que obt totes les denominacions principals i arees temtiques de les fitxes
    *       d'un diccionari determinat, mitjanant el mtode findAllDenominationsAndTematicAreas.
    *       @param idDiccionari:    Identificador del diccionari.
    *       @return Vector:         Vector amb les denominacions principals i les arees temtiques de totes les
    *                               fitxes del diccionari. 
    */
    public Vector getAllDenominacionsPrincipalsAndTematicAreas(String idDiccionari, int rowIni, int maxRows) throws DataException
    {

         Vector listDenominacions=null;
         try
         {
            Denominacions denominacions=new Denominacions();                     
            listDenominacions=denominacions.findAllPrincipalDenominationsAndTematicAreas(idDiccionari, rowIni,maxRows);
         }
         catch(Exception e)
         {
             e.printStackTrace();
             throw new DataException(e.getMessage());
         }
         
         return listDenominacions;
    }
    
    /*
     *       Mtode que afegeix una observaci a una fitxa .
     */
    public void altaObservacions(String idFitxa, String observacions) throws DataException
    {       
        try
        {
            Connection con=DBConnect.getConnectionTransactional();
            Fitxes fitxa=new Fitxes();
            Fitxes fitxaSelected=(Fitxes)fitxa.findByPrimaryKey(idFitxa);
            fitxaSelected.setObservacions(observacions);
            fitxaSelected.flush(con);
        }
        catch(Exception e)
        {
            throw new DataException(e.getMessage());
        }
    }
    
     public Element getAllFitxesToXML(String idDiccionari) throws DataException
     {
         Document doc=new Document();
         Element elem=new Element("DadesFitxes");
         try
         {
            Fitxes newFitxa=new Fitxes();
            Fitxes fitxa=new Fitxes();
            String sql=" ID_DICCIONARI='"+idDiccionari+"'";
            List listFitxes=newFitxa.findByGenericWhere(sql);
            for(int i=0;i<listFitxes.size();++i)
            {
                fitxa = (Fitxes)listFitxes.get(i);                
                elem.addElement(fitxa.toXml());
            }
            
            doc.setRoot(elem);
            //System.out.println("DOC:"+doc.toString());
            return doc.getRoot();
         }
         catch(Exception e)
         {
             e.printStackTrace();
             throw new DataException(e.getMessage());
         }       
    }  

    public void deleteAllFitxes() throws DataException
    {
        String sql="delete from CO_FITXES";
        try 
        {
            com.termcat.bd.beans.BDUtil.execute(sql);        
        } 
        catch (Exception e) 
        {
            throw new DataException(e.getMessage());
        }        
    }      

    public Fitxes getFitxaInfo(String idFitxa) throws DataException
    {
       Fitxes fitxa=new Fitxes();
        try 
        {
            Fitxes fitxaSelected=(Fitxes)fitxa.findByPrimaryKey(idFitxa);  
            return fitxaSelected;
        } 
        catch (FinderException ex) 
        {          
            ex.printStackTrace();
            throw new DataException(ex.getMessage());
        }               
    }
}