/*
 * DenominationManager.java
 *
 * Created on 7 de agosto de 2007, 17:12
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.termcat.business.terminologic.denominations;

import com.termcat.bd.beans.Denominacions;
import com.termcat.bd.beans.Fitxes;
import com.termcat.bd.exceptions.FinderException;
import com.termcat.business.exceptions.DenominationException;
import com.termcat.business.reports.veurefitxa.BeanVeureFitxaDenominacio;
import com.termcat.business.terminologic.FitxaElement;
import com.termcat.layout.FunctionalModule;
import com.termcat.layout.commons.lists.ItemListCustom;
import com.termcat.util.UIDGenerator;
import electric.xml.Child;
import electric.xml.Document;
import electric.xml.Element;
import electric.xml.Node;
import java.sql.Connection;
import java.util.List;
import java.util.Vector;
import com.termcat.bd.utils.DBConnect;

/**
 *
 * @author Mena
 */
public class DenominationManager {
    
    private static DenominationManager manager;
    
    /** Creates a new instance of DenominationManager */
    public DenominationManager() {
    }
     
    public static DenominationManager getManager()
    {
        if(manager==null)
        {
            manager = new DenominationManager();
        }
     
       return manager;
    }

    /*
     *       Mtode que obt una llista amb totes les denominacions d'una determinada fitxa.
     *      @param idFitxa: Identificador de la fitxa.
     */
    public List getAllDenominationsOfFitxa(String idFitxa)
    {
        Vector l=null;
        try
        {
            /*String sql="ID_FITXA='"+idFitxa+"'";
            denominacio=new Denominacions();
            l=denominacio.findByGenericWhere(sql);*/
            //modificacion para tener ordenadas las denoinaciones
            Vector<FitxaElement> sortDenos = Fitxes.getAllDenosFromFitxa(idFitxa,"void");
            int numDenos = sortDenos.size();
            l = new Vector(numDenos);
            for(int i=0;i<numDenos;i++){
                String denoId = sortDenos.get(i).getIdElement();
                Denominacions denoBean = new Denominacions();
                denoBean.findByPrimaryKey(denoId);
                l.add(denoBean);
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
                
        return l;
    }

    public void altaDenominacio(String idDenominacio, String idDiccionari, String idFitxa, String idLlengua, String termeNormal, String termeRtf, String idJerarquia, String idCategoria, String idMarca) throws DenominationException 
    {
        try
        {
            Connection con=DBConnect.getConnectionTransactional();
            Denominacions denominacio=new Denominacions();
            denominacio.setIdDenominacio(idDenominacio);
            denominacio.setIdDiccionari(idDiccionari);
            denominacio.setIdLlengua(idLlengua);
            denominacio.setTermeNormal(termeNormal);
            denominacio.setTermeRtf(termeRtf);
            if (idJerarquia!=null)
                denominacio.setIdJerarquia(idJerarquia);
            if (idCategoria!=null)                
                denominacio.setIdCategoria(idCategoria);
            if (idMarca!=null)
                denominacio.setIdMarca(idMarca);
            denominacio.setIdFitxa(idFitxa);
            denominacio.create(con);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new DenominationException(e.getMessage());
        }        
    }
    
    /*
     *  Mtode que inserta una nova denominaci a la BD.
     *  
     *  @param idDiccionari:    Identificador del diccionari al que pertany la fitxa.
     *  @param idFitxa:         Identificador de la fitxa a la que pertany.
     *  @param idLlengua:       Identificador de la llengua.
     *  @param termoNormal:     Contingut de la denominaci sense format.
     *  @param termeRtf:        Contingut de la denominaci en format rtf.
     *  @param idCategoria:     Identificador de la categoria a la que pertany.
     *  @param idJerarquia:     Identificador de la jerarquia a la que pertany.
     *  @param idMarca:         Identificador de la marca de ponderaci a la que pertany.
     */
    public String altaDenominacio(String idDiccionari,String idFitxa,String idLlengua, String termeNormal, String termeRtf,String idJerarquia, String idCategoria, String idMarca) throws DenominationException
    {
        String idDenominacio=null;
        try
        {
            Connection con=DBConnect.getConnectionTransactional();
            Denominacions denominacio=new Denominacions();
            idDenominacio=UIDGenerator.generateUniqueId();
            denominacio.setIdDenominacio(idDenominacio);
            denominacio.setIdDiccionari(idDiccionari);
            denominacio.setIdLlengua(idLlengua);
            denominacio.setTermeNormal(termeNormal);
            denominacio.setTermeRtf(termeRtf);
            denominacio.setIdJerarquia(idJerarquia);
            denominacio.setIdCategoria(idCategoria);
            denominacio.setIdMarca(idMarca);
            denominacio.setIdFitxa(idFitxa);
            denominacio.create(con);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new DenominationException(e.getMessage());
        }
        return idDenominacio;
    }
   
     /*
     *       Mtode que esborra una denominaci.
     *      @param idDenominacio:     Identificador de la denominaci.
     */
    public void deleteDenominacio(String idDenominacio) throws DenominationException 
    {
        try
        {
            Connection con=DBConnect.getConnectionTransactional();
            Denominacions denominacio=new Denominacions();
            Denominacions denominacioSelected=(Denominacions)denominacio.findByPrimaryKey(idDenominacio);
            denominacioSelected.remove(con);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new DenominationException(e.getMessage());
        }
    }
    

    /*
     *  Mtode que actualitza els valors d'una denominaci de la BD.
     * 
     *  @param idDenominaci:   Identificador de la denominaci.
     *  @param idLlengua:       Identificador de la llengua.
     *  @param termeNormal:     Contingut de la denominaci sense format.
     *  @param termeRtf:        Contingut de la denominaci en format rtf.
     *  @param idCategoria:     Identificador de la categoria a la que pertany.
     *  @param idJerarquia:     Identificador de la jerarquia a la que pertany.
     *  @param idMarca:         Identificador de la marca de ponderaci a la que pertany.
     */
    public void updateDenominacio(String idDenominacio,String idLlengua, String termeNormal, String termeRtf,String idJerarquia, String idCategoria, String idMarca) throws DenominationException
    {
        try
        {
            Connection con=DBConnect.getConnectionTransactional();
            Denominacions denominacio=new Denominacions();
            Denominacions denominacioSelected=(Denominacions)denominacio.findByPrimaryKey(idDenominacio);
            denominacioSelected.setIdLlengua(idLlengua);
            denominacioSelected.setTermeNormal(termeNormal);
            denominacioSelected.setTermeRtf(termeRtf);
            denominacioSelected.setIdJerarquia(idJerarquia);
            denominacioSelected.setIdCategoria(idCategoria);
            denominacioSelected.setIdMarca(idMarca);
            denominacioSelected.flush(con);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new DenominationException(e.getMessage());
        }
        
    }
    
      public Element getAllDenominationsToXML(String idDiccionari)
     {
         Document doc=new Document();
         Element elem=new Element("DadesDenominacions");
         String sql=" ID_DICCIONARI='"+idDiccionari+"'";
         
         try
         {
            Denominacions newDenomination=new Denominacions();
            Denominacions denomination=new Denominacions();

            List listDenominations=newDenomination.findByGenericWhere(sql);
            for(int i=0;i<listDenominations.size();++i)
            {
                denomination = (Denominacions)listDenominations.get(i);
                elem.addElement(denomination.toXml());               
            }            
            doc.setRoot(elem);
         }
         catch(Exception e)
         {
             e.printStackTrace();
         }

         return doc.getRoot();
    }
      
    
    public String getRTFDenomination(String id) throws DenominationException 
    {
       String descriptionRTF="";  
       try
        {
            Denominacions deno=new Denominacions();
            Denominacions denoSelected=(Denominacions)deno.findByPrimaryKey(id);
            descriptionRTF=denoSelected.getTermeRtf();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            System.out.println("MESSAGE: "+e.getMessage());

            throw new DenominationException(e.getMessage());
        }        
       
       return descriptionRTF;
    }      

    public void deleteAllDenominacions() throws DenominationException
    {
        String sql="delete from CO_DENOMINACIONS";
        try 
        {
            com.termcat.bd.beans.BDUtil.execute(sql);        
        } 
        catch (Exception e) 
        {
            throw new DenominationException(e.getMessage());
        }        
    }    

    public Vector getAllDenominationsOfFitxaAndFonts(String idDiccionari,String id) throws FinderException 
    {
        Vector<BeanVeureFitxaDenominacio> v = new Vector<BeanVeureFitxaDenominacio>();  
        
        Denominacions deno=new Denominacions();
        Vector<BeanVeureFitxaDenominacio> denoInfo=deno.findDenominationInfoAndFonts(idDiccionari,id);
        
        // Construim el format correcte per a les fonts : (font1;font2;...;fontN)
        v = FunctionalModule.createFontDenominacions(denoInfo);       
         
        return v;
    }
}