/*
 * ToolManager.java
 *
 * Created on 19 de noviembre de 2007, 16:20
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.termcat.business.tools;

import com.termcat.bd.beans.Denominacions;
import com.termcat.bd.beans.Diccionaris;
import com.termcat.bd.exceptions.FinderException;
import com.termcat.bd.utils.DBConnect;
import com.termcat.business.areas.AreaManager;
import com.termcat.business.dictionary.DictionaryManager;
import com.termcat.business.exceptions.AreaException;
import com.termcat.business.exceptions.CategoryException;
import com.termcat.business.exceptions.DataException;
import com.termcat.business.exceptions.DefinitionException;
import com.termcat.business.exceptions.DenominationException;
import com.termcat.business.exceptions.DictionaryException;
import com.termcat.business.exceptions.FontException;
import com.termcat.business.exceptions.HyerarchyException;
import com.termcat.business.exceptions.LanguageException;
import com.termcat.business.exceptions.MarkException;
import com.termcat.business.exceptions.NoteException;
import com.termcat.business.exceptions.ToolException;
import com.termcat.business.maintenance.category.CategoryManager;
import com.termcat.business.maintenance.font.FontManager;
import com.termcat.business.maintenance.hyerarchy.HyerarchyManager;
import com.termcat.business.maintenance.language.LanguageManager;
import com.termcat.business.maintenance.mark.MarkManager;
import com.termcat.business.terminologic.contexts.ContextManager;
import com.termcat.business.terminologic.data.DataManager;
import com.termcat.business.terminologic.definitions.DefinitionManager;
import com.termcat.business.terminologic.denominations.DenominationManager;
import com.termcat.business.terminologic.notes.NoteManager;
import com.termcat.layout.FunctionalModule;
import com.termcat.layout.MainFrame;
import com.termcat.synchronizer.exceptions.ExportException;
import com.termcat.util.literales.Literales;
import electric.xml.Document;
import electric.xml.Element;
import electric.xml.Elements;
import electric.xml.ParseException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.sql.SQLException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
import javax.swing.JLabel;

/**
 *
 * @author Mena
 */
public class ToolManager 
{
    private MainFrame menuFrame=null;
    
    private static ToolManager manager;
    
    /** Creates a new instance of DenominationManager */
    private ToolManager(MainFrame menuFrame)
    {
        this.menuFrame=menuFrame;
    }
     
    public static ToolManager getManager(MainFrame menuFrame)
    {
        if(manager==null)
        {
            manager = new ToolManager(menuFrame);
        }
     
       return manager;
    }
    
    public void importDadesFromXML(File file, JLabel diccionariName) throws ToolException  
    {
        String fileName=file.getAbsolutePath();
        System.out.println("[importDadesFromXML] Important dades del fitxer '"+fileName+"' ...");
        Document xmlDoc;
        
        try 
        {
            ZipInputStream fileIn = new ZipInputStream(new FileInputStream(fileName));
            fileIn.getNextEntry();
            xmlDoc = new Document(fileIn);
            Element diccionari=xmlDoc.getElement(DICCIONARI);
            
            // Fer el init de la transaccio.
            java.sql.Connection ct=DBConnect.getConnectionTransactional(); 
            
            // Esborrem el contingut de les taules del diccionari anterior
            truncateAllTables();           
            
            // Import de les dades
            importDadesDiccionari(diccionari);
            importDadesFitxes(diccionari);
            importDadesArbreCamp(diccionari);
            importAreesPerFitxa(diccionari);
            importDadesLlengues(diccionari);
            importDadesDefinicions(diccionari);
            importDadesNotes(diccionari);
            importDadesContextes(diccionari);
            importDadesMarques(diccionari);
            importDadesJerarquies(diccionari);
            importDadesCategories(diccionari);
            importDadesDenominacions(diccionari);
            importDadesFonts(diccionari);
            
            // Fer el commit
            DBConnect.getConnectionTransactional().commit();
            
            // Fer l'init del diccionari
            menuFrame.initMainFrame();
            
           // Actualitzem el nom del diccionari a la pantalla
           //diccionariName.setText(menuFrame.getNomCurrentDictionary());
            
            fileIn.close();
            System.out.println("[thread] import done!");
        }
        catch(ParseException ee){ //no es un fichero XML --> fin , no se hace rollback por que aun no se ha iniciado la transaccion
            throw new ToolException(Literales.getTextForMessage("Excepcio_Importacio"));    
        }
        catch (Exception e) 
        {
            e.printStackTrace();
            // Fem el rollback            
            try 
            {
                DBConnect.getConnectionTransactional().rollback();
                throw new ToolException(Literales.getTextForMessage("ImportKO"));
            }
            catch (SQLException ex) 
            {
                ex.printStackTrace();
                throw new ToolException(Literales.getTextForMessage("ImportKO"));  
                
                // En cas de no poder fer el rollback, s'hauria de esborrar tot el contingut de ls BD una altre vegada 
                // per a que no quedes inconsistent l'estat de la mateixa.
            }                       
        }
        finally{
           DBConnect.putConnectionTransactional(); 
        }
    }


    private void importDadesDiccionari(Element diccionari) throws DictionaryException 
    {
       
        String id="",observacions="", descripcio="", nom="";
        Element diccionaris=diccionari.getElement(DICCIONARIS);
        //printElement("DIC",diccionaris);
        
        id=diccionaris.getTextString(ID_DICCIONARI);
        nom=diccionaris.getTextString(NOM);
        descripcio=diccionaris.getTextString(DESCRIPCIO);
        observacions=diccionaris.getTextString(OBSERVACIONS);
               
        System.out.println("[importDadesDiccionari] ID:"+id+" NOM:"+nom+" DESC:"+descripcio+" OBS:"+observacions);

        // Fem els setters a la Base de dades
        DictionaryManager.getManager().altaDiccionari(id,nom,descripcio,observacions);
    }

    // ----------------------------------------------------------------------------------------------------------
    // Constants pels tags de l'XML d'importaci --> Potser posarles en un fitxer de constants "GLOBAL" que implementi el FunctionalModule
    // ----------------------------------------------------------------------------------------------------------    
    // Arrel
    private final String DICCIONARI="Diccionari";

    // 1er. Nivell
    private final String DICCIONARIS="Diccionaris";
    private final String DADES_FITXES="DadesFitxes";
    private final String DADES_ARBRE_CAMP="DadesArbreCamp";
    private final String AREES_FITXA="AreesPerFitxa";
    private final String DADES_LLENGUES="DadesLlengues";
    private final String DADES_DEFINICIONS="DadesDefinicions";
    private final String DADES_NOTES="DadesNotes";
    private final String DADES_CONTEXTES="DadesContextes";
    private final String DADES_MARQUES="DadesMarques";
    private final String DADES_CATEGORIES="DadesCategories";
    private final String DADES_JERARQUIES="DadesJerarquies";
    private final String DADES_DENOMINACIONS="DadesDenominacions";
    private final String DADES_FONTS="DadesFonts";
    
    // 2on. Nivell: Nom dels beans.
    private final String FITXES="Fitxes";
    private final String ARBRE_CAMP="ArbreCamp";
    private final String AREES_FITXES="AreesFitxes";
    private final String LLENGUES="Llengues";
    private final String LLENGUES_DICCIONARI="LlenguesDiccionari";
    private final String DEFINICIONS="Definicions";
    private final String NOTES="Notes";
    private final String CONTEXTE="Context";
    private final String MARQUES_P="MarquesP";
    private final String CATEGORIES="Categories";
    private final String JERARQUIES="Jerarquies";
    private final String DENOMINACIONS="Denominacions";
    private final String LLISTAT_FONTS="LlistatFonts";
    private final String DADES_FONTS_DENOMINACIONS="DadesFontsDenominacions";
    private final String DADES_FONTS_DEFINICIONS="DadesFontsDefinicions";
    private final String DADES_FONTS_CONTEXTES="DadesFontsContextes";
    private final String DADES_FONTS_NOTES="DadesFontsNotes";
    
    // 3er. Nivell: Nom dels beans
    private final String FONTS="Fonts";
    private final String FONTS_DENOMINACIONS="FontsDenominacions";
    private final String FONTS_DEFINICIONS="FontsDefinicions";
    private final String FONTS_CONTEXTOS="FontsContextos";
    private final String FONTS_NOTES="FontsNotes";
 
    
    // Atributs dels beans

    // Generics
    private final String NOM="nom";
    private final String DESCRIPCIO="descripcio";
    private final String OBSERVACIONS="observacions";
    private final String PRIORITAT="prioritat";
    
    // Diccionari
    private final String ID_DICCIONARI="idDiccionari";
    
    // Fitxes
    private final String ID_FITXA="idFitxa";
    private final String IMAGE_PATH="imagePath";

    // Arees Tematiques
    private final String ID_AREA="idArea";
    private final String ID_PARE="idPare";
    private final String NOM_AREA="nomArea";

    // Llengues
    private final String ID_LLENGUA="idLlengua";
    private final String CODI_LLENGUA="codiLlengua";

    // Definicions
    private final String ID_DEFINICIO="idDefinicio";
    private final String DEFINICION_NORMAL="definicionNormal";
    private final String DEFINICION_RTF="definicionRtf";
    
    // Notes
    private final String ID_NOTA="idNota";
    private final String NOTA_NORMAL="notaNormal";
    private final String NOTA_RTF="notaRtf";
    
    // Context
    private final String ID_CONTEXT="idContext";
    private final String CONTEXT_RTF="contextRtf";
    private final String CONTEXT_NORMAL="contextNormal";
    
    // Marques
    private final String ID_MARCA="idMarca";
    private final String CODI_MARCA="codiMarca_p";
    
    // Categories
    private final String ID_CATEGORIA="idCategoria";
    private final String CODI_CATEGORIA="codiCategoria";
    
    // Jerarquies
    private final String ID_JERARQUIA="idJerarquia";
    private final String CODI_JERARQUIA="codiJerarquia";
    
    // Denominacions
    private final String ID_DENOMINACIO="idDenominacio";
    private final String TERME_NORMAL="termeNormal";
    private final String TERME_RTF="termeRtf";
    
    // Fonts
    private final String ID_FONT="idFont";
    private final String CODI_FONT="codiFont";
               

    private void importDadesFitxes(Element diccionari) throws DataException 
    {
        String idFitxa="", idDiccionari="", observacions="", imagePath="";
        Element dadesFitxes=diccionari.getElement(DADES_FITXES);
        Elements fitxesList=(Elements) dadesFitxes.getElements(FITXES);
        if(fitxesList.size()>0)
        {
            while(fitxesList.hasMoreElements())
            {
                Element e=fitxesList.next();
                idFitxa=e.getTextString(ID_FITXA);
                idDiccionari=e.getTextString(ID_DICCIONARI);
                observacions=e.getTextString(OBSERVACIONS);
                imagePath=e.getTextString(IMAGE_PATH);
                System.out.println("[importDadesFitxa] ID:"+idFitxa+" Observacions:"+observacions);
                
                // Fem el insert a la BD.
                DataManager.getManager().altaFitxa(idFitxa,idDiccionari,observacions,imagePath);
            }
        }        
    }

    private void importDadesArbreCamp(Element diccionari) throws AreaException 
    {
        String idArea="", idDiccionari="", idPare="", nomArea="";
        Element dadesArbreCamp=diccionari.getElement(DADES_ARBRE_CAMP);
        Elements arbreCampList=(Elements) dadesArbreCamp.getElements(ARBRE_CAMP);
        if(arbreCampList.size()>0)
        {
            while(arbreCampList.hasMoreElements())
            {
                Element e=arbreCampList.next();
                idArea=e.getTextString(ID_AREA);
                idDiccionari=e.getTextString(ID_DICCIONARI);
                idPare=e.getTextString(ID_PARE);
                nomArea=e.getTextString(NOM_AREA);
                System.out.println("[importDadesArbreCamp] ID:"+idArea+" nomArea:"+nomArea+" IDPare:"+idPare);
                
                // Fem el insert a la BD.
                AreaManager.getManager().altaArea(idDiccionari,idArea,idPare,nomArea);
            }
        }          
    }

    private void importAreesPerFitxa(Element diccionari) throws AreaException 
    {
        String idArea="", idFitxa="";
        Element areesFitxa=diccionari.getElement(AREES_FITXA);
        Elements areesFitxaList=(Elements) areesFitxa.getElements(AREES_FITXES);
        if(areesFitxaList.size()>0)
        {
            while(areesFitxaList.hasMoreElements())
            {
                Element e=areesFitxaList.next();
                idArea=e.getTextString(ID_AREA);             
                idFitxa=e.getTextString(ID_FITXA);
               
                System.out.println("[importAreesPerFitxa] IDArea:"+idArea+" idFitxa:"+idFitxa);
                
                // Fem el insert a la BD.
                AreaManager.getManager().altaAreaFitxa(idArea,idFitxa);
            }               
        }        
    }

    private void importDadesLlengues(Element diccionari) throws LanguageException 
    {
        Element dadesLlengues=diccionari.getElement(DADES_LLENGUES);        
        
        importLlengues(dadesLlengues);
        importLLenguesDiccionari(dadesLlengues);
    }

    private void importLLenguesDiccionari(Element dadesLlengues) throws LanguageException 
    {
        String idDiccionari="",idLlengua="";
        int  prioritat=-1;
        Elements llenguesDiccionariList=dadesLlengues.getElements(LLENGUES_DICCIONARI);
        if(llenguesDiccionariList.size()>0)
        {
            while(llenguesDiccionariList.hasMoreElements())
            {
                Element e=llenguesDiccionariList.next();
                //System.out.println("LL.diccionari List: "+e.toString());
                idDiccionari=e.getTextString(ID_DICCIONARI);             
                idLlengua=e.getTextString(ID_LLENGUA);
                prioritat=e.getInt(PRIORITAT);
               
                System.out.println("[importLLenguesDiccionari] IdLlengua:"+idLlengua+" Prioritat:"+prioritat);
                
                // Fem el insert a la BD.
                LanguageManager.getManager().altaLlenguesDiccionari(idDiccionari,idLlengua,prioritat);                
            }               
        }        
        
        
    }

    private void importLlengues(Element dadesLlengues) throws LanguageException 
    {
        String idLlengua="",codiLlengua="",nom="";
        Elements llenguesList=dadesLlengues.getElements(LLENGUES);
        
        if(llenguesList.size()>0)
        {
            while(llenguesList.hasMoreElements())
            {
                Element e=llenguesList.next();
                idLlengua=e.getTextString(ID_LLENGUA);
                codiLlengua=e.getTextString(CODI_LLENGUA);
                nom=e.getTextString(NOM);

                System.out.println("[importLlengues] NomLlengua:"+idLlengua+" Nom:"+nom);
                
                // Fem el insert a la BD.
                LanguageManager.getManager().altaLlengua(idLlengua,codiLlengua,nom);                                               
            }            
        }        
    }
    
    private void importDadesDefinicions(Element diccionari) throws DefinitionException 
    {
        String idLlengua="",idDefinicio="",idDiccionari="",idFitxa="",defNormal="",defRtf="";
        Element dadesDefinicions=diccionari.getElement(DADES_DEFINICIONS);
        Elements definicionsList=dadesDefinicions.getElements(DEFINICIONS);
        if(definicionsList.size()>0)
        {
            while(definicionsList.hasMoreElements())
            {
                Element e=definicionsList.next();
                idLlengua=e.getTextString(ID_LLENGUA);
                idDefinicio=e.getTextString(ID_DEFINICIO);
                idDiccionari=e.getTextString(ID_DICCIONARI);
                idFitxa=e.getTextString(ID_FITXA);
                defNormal=e.getTextString(DEFINICION_NORMAL);
                //defRtf=rtf2html.convertRTF2HTML(e.getString(DEFINICION_RTF));
                defRtf=e.getTextString(DEFINICION_RTF);

                System.out.println("[importDadesDefinicions] idDef:"+idDefinicio+" def:"+defNormal);
                // Fem el insert a la BD.
                DefinitionManager.getManager().altaDefinicio(idDefinicio,idDiccionari,idFitxa,idLlengua,defNormal,defRtf);
            }            
        }        
    }

    private void importDadesNotes(Element diccionari) throws NoteException 
    {
        String idLlengua="",idNota="",idDiccionari="",idFitxa="",notaNormal="",notaRtf="";
        Element dadesNotes=diccionari.getElement(DADES_NOTES);
        Elements notesList=dadesNotes.getElements(NOTES);
        if(notesList.size()>0)
        {
            while(notesList.hasMoreElements())
            {
                Element e=notesList.next();
                idLlengua=e.getTextString(ID_LLENGUA);
                idNota=e.getTextString(ID_NOTA);
                idDiccionari=e.getTextString(ID_DICCIONARI);
                idFitxa=e.getTextString(ID_FITXA);
                notaNormal=e.getTextString(NOTA_NORMAL);
                //notaRtf=rtf2html.convertRTF2HTML(e.getString(NOTA_RTF));
                notaRtf=e.getTextString(NOTA_RTF);

                System.out.println("[importDadesNotes] idNota:"+idNota+" nota:"+notaNormal);
                // Fem el insert a la BD.
                NoteManager.getManager().altaNota(idNota,idDiccionari,idFitxa,idLlengua,notaNormal,notaRtf);
            }            
        }        
    }
    
    private void importDadesContextes(Element diccionari) throws Exception
    {
        String idLlengua="",idContexte="",idDiccionari="",idFitxa="",contextRtf="",context="";
        Element dadesContextes=diccionari.getElement(DADES_CONTEXTES);
        Elements contexteList=dadesContextes.getElements(CONTEXTE);
        if(contexteList.size()>0)
        {
            while(contexteList.hasMoreElements())
            {
                Element e=contexteList.next();
                idLlengua=e.getTextString(ID_LLENGUA);
                idContexte=e.getTextString(ID_CONTEXT);
                idDiccionari=e.getTextString(ID_DICCIONARI);
                idFitxa=e.getTextString(ID_FITXA);
                contextRtf=e.getTextString(CONTEXT_RTF);
                context=e.getTextString(CONTEXT_NORMAL);//FunctionalModule.convertRTF2Text(contextRtf);
                System.out.println("[importDadesContextes] idContext:"+idContexte+" context:"+context);
                // Fem el insert a la BD.
                ContextManager.getManager().altaContext(idContexte,idDiccionari,idFitxa,idLlengua,context,contextRtf);
            }            
        }        
    }

    private void importDadesMarques(Element diccionari) throws MarkException 
    {
        String idMarca="",codi="",descripcio="";
        Element dadesMarques=diccionari.getElement(DADES_MARQUES);
        Elements marquesList=dadesMarques.getElements(MARQUES_P);
        if(marquesList.size()>0)
        {
            while(marquesList.hasMoreElements())
            {
                Element e=marquesList.next();

                idMarca=e.getTextString(ID_MARCA);
                codi=e.getTextString(CODI_MARCA);
                descripcio=e.getTextString(DESCRIPCIO);
                
                System.out.println("[importDadesMarques] codi:"+codi+" descripcio:"+descripcio);
                // Fem el insert a la BD.
                MarkManager.getManager().altaMarca(idMarca,codi,descripcio);
            }            
        }        
        
    }

    private void importDadesCategories(Element diccionari) throws CategoryException 
    {
        String idCategoria="",codi="",descripcio="";
        int prioritat=-1;
        Element dadesCategories=diccionari.getElement(DADES_CATEGORIES);
        Elements categoriesList=dadesCategories.getElements(CATEGORIES);
        if(categoriesList.size()>0)
        {
            while(categoriesList.hasMoreElements())
            {
                Element e=categoriesList.next();
                idCategoria=e.getTextString(ID_CATEGORIA);
                codi=e.getTextString(CODI_CATEGORIA);
                descripcio=e.getTextString(DESCRIPCIO);
                prioritat=e.getInt(PRIORITAT);
                System.out.println("[importDadesCategories] codi:"+codi+" descripcio:"+descripcio+" prioritat:"+prioritat);
                // Fem el insert a la BD.
                CategoryManager.getManager().altaCategoria(idCategoria,codi,descripcio,prioritat);
            }            
        }        
        
    }
    
    private void importDadesJerarquies(Element diccionari) throws HyerarchyException
    {
        String idJerarquia="",codi="",descripcio="";
        int prioritat=-1;
        Element dadesJerarquies=diccionari.getElement(DADES_JERARQUIES);
        Elements jerarquiesList=dadesJerarquies.getElements(JERARQUIES);
        if(jerarquiesList.size()>0)
        {
            while(jerarquiesList.hasMoreElements())
            {
                Element e=jerarquiesList.next();
               // System.out.println("Jerarquies: "+e.toString());
                idJerarquia=e.getTextString(ID_JERARQUIA);
                codi=e.getTextString(CODI_JERARQUIA);
                descripcio=e.getTextString(DESCRIPCIO);
                prioritat=e.getInt(PRIORITAT);
                
                System.out.println("[importDadesJerarquies] codi:"+codi+" descripcio:"+descripcio+" prioritat:"+prioritat);
                // Fem el insert a la BD.
                HyerarchyManager.getManager().altaJerarquia(idJerarquia,codi,descripcio,prioritat);
            }            
        }        
        
    }  
    
    private void importDadesDenominacions(Element diccionari) throws DenominationException 
    {
        String idLlengua="",idDenominacio="",idDiccionari="",idFitxa="",termeNormal="",termeRtf="",idJerarquia="",idCategoria="",idMarca="";
        Element dadesDenominacions=diccionari.getElement(DADES_DENOMINACIONS);
        Elements denominacionsList=dadesDenominacions.getElements(DENOMINACIONS);
        if(denominacionsList.size()>0)
        {
            while(denominacionsList.hasMoreElements())
            {
                Element e=denominacionsList.next();
                idLlengua=e.getTextString(ID_LLENGUA);
                idDenominacio=e.getTextString(ID_DENOMINACIO);
                idDiccionari=e.getTextString(ID_DICCIONARI);
                idFitxa=e.getTextString(ID_FITXA);
                termeNormal=e.getTextString(TERME_NORMAL);
                //termeRtf=rtf2html.convertRTF2HTML(e.getString(TERME_RTF));
                termeRtf=e.getTextString(TERME_RTF);
                idCategoria=e.getTextString(ID_CATEGORIA);                
                idJerarquia=e.getTextString(ID_JERARQUIA);
                if (idJerarquia ==null)
                    idJerarquia="";              
                idMarca=e.getTextString(ID_MARCA);                
                if (idMarca==null)
                    idMarca="";

                System.out.println("[importDadesDenominacions] idDenom:"+idDenominacio+" terme:"+termeNormal);
                // Fem el insert a la BD.
                DenominationManager.getManager().altaDenominacio(idDenominacio,idDiccionari,idFitxa,idLlengua,termeNormal,termeRtf,idJerarquia,idCategoria,idMarca);
            }            
        }        
    }

    private void importDadesFonts(Element diccionari) throws FontException, FinderException 
    {
        Element dadesFonts=diccionari.getElement(DADES_FONTS);        
        
        importLlistatFonts(dadesFonts);
        importDadesFontsDenominacions(dadesFonts);        
        importDadesFontsDefinicions(dadesFonts);        
        importDadesFontsContextes(dadesFonts);                
        importDadesFontsNotes(dadesFonts);                        
        
    }

    private void importLlistatFonts(Element dadesFonts) throws FontException  
    {
        String idFont="",codiFont="",descripcio="";
        Element fonts=dadesFonts.getElement(LLISTAT_FONTS);
        Elements fontsList=fonts.getElements(FONTS);
        if(fontsList.size()>0)
        {
            while(fontsList.hasMoreElements())
            {
                Element e=fontsList.next();
                idFont=e.getTextString(ID_FONT);              
                codiFont=e.getTextString(CODI_FONT);
                descripcio=e.getTextString(DESCRIPCIO);
                
                System.out.println("[importFonts] CodiFont:"+codiFont+" descripcio:"+descripcio);
                
                // Fem el insert a la BD.
                FontManager.getManager().altaFont(idFont,codiFont,descripcio);                                               
            }            
        }              
    }

    private void importDadesFontsDenominacions(Element dadesFonts) throws FontException, FinderException 
    {
        String idDenominacio="",idFont="";

        
        Element fontsDenominacions=dadesFonts.getElement(DADES_FONTS_DENOMINACIONS);
        Elements fontsDenominacionsList=fontsDenominacions.getElements(FONTS_DENOMINACIONS);
        
        if(fontsDenominacionsList.size()>0)
        {
            while(fontsDenominacionsList.hasMoreElements())
            {
                Element e=fontsDenominacionsList.next();
                idDenominacio=e.getTextString(ID_DENOMINACIO);             
                idFont=e.getTextString(ID_FONT);
               
                System.out.println("[importFontsLLenguesDiccionari > DENOMINACIO] idDenominacio:"+idDenominacio+" idFont:"+idFont);
                
                // Fem el insert a la BD.
                FontManager.getManager().altaFontDenominacio(idDenominacio,idFont);
            }               
        }        
    }

    private void importDadesFontsDefinicions(Element dadesFonts) throws FontException 
    {
        String idDefinicio="",idFont="";

        
        Element fontsDefinicions=dadesFonts.getElement(DADES_FONTS_DEFINICIONS);
        Elements fontsDefinicionsList=fontsDefinicions.getElements(FONTS_DEFINICIONS);
        if(fontsDefinicionsList.size()>0)
        {
            while(fontsDefinicionsList.hasMoreElements())
            {
                Element e=fontsDefinicionsList.next();
                idDefinicio=e.getTextString(ID_DEFINICIO);             
                idFont=e.getTextString(ID_FONT);
               
                System.out.println("[importFontsLLenguesDiccionari > DEFINICIO] idDefinicio:"+idDefinicio+" idFont:"+idFont);
                
                // Fem el insert a la BD.
                FontManager.getManager().altaFontDefinicions(idDefinicio,idFont);
            }               
        }        
    }    
    
     private void importDadesFontsContextes(Element dadesFonts) throws FontException 
    {
        String idCtxt="",idFont="";
       
        Element fontsContextes=dadesFonts.getElement(DADES_FONTS_CONTEXTES);
        Elements fontsContextesList=fontsContextes.getElements(FONTS_CONTEXTOS);        
        if(fontsContextesList.size()>0)
        {
            while(fontsContextesList.hasMoreElements())
            {
                Element e=fontsContextesList.next();
                idCtxt=e.getTextString(ID_CONTEXT);             
                idFont=e.getTextString(ID_FONT);
               
                System.out.println("[importFontsLLenguesDiccionari > CONTEXTES] idCtxt:"+idCtxt+" idFont:"+idFont);
                
                // Fem el insert a la BD.
                FontManager.getManager().altaFontContextes(idCtxt,idFont);
            }               
        }        
    } 
     
    
     private void importDadesFontsNotes(Element dadesFonts) throws FontException 
    {
        String idNota="",idFont="";
       
        Element fontsNotes=dadesFonts.getElement(DADES_FONTS_NOTES);
        Elements fontsNotesList=fontsNotes.getElements(FONTS_NOTES);
        if(fontsNotesList.size()>0)
        {
            while(fontsNotesList.hasMoreElements())
            {
                Element e=fontsNotesList.next();
                idNota=e.getTextString(ID_NOTA);             
                idFont=e.getTextString(ID_FONT);
               
                System.out.println("[importFontsLLenguesDiccionari > NOTES] idNota:"+idNota+" idFont:"+idFont);
                
                // Fem el insert a la BD.
                FontManager.getManager().altaFontNotes(idNota,idFont);
            }               
        }        
    }      

    private void truncateAllTables() throws Exception
    {
    
            // Esborrem les 'fulles'
            AreaManager.getManager().deleteAllAreesFitxes();
            AreaManager.getManager().deleteAllArbreCamps();                        
                       
            FontManager.getManager().deleteAllFontsContextos();
            FontManager.getManager().deleteAllFontsDefinicions();
            FontManager.getManager().deleteAllFontsDenominacions();
            FontManager.getManager().deleteAllFontsNotes();
            
            // Esborrer els penultim nodes.
            DenominationManager.getManager().deleteAllDenominacions();            

            ContextManager.getManager().deleteAllContextes();
            DefinitionManager.getManager().deleteAllDefinitions();
            NoteManager.getManager().deleteAllNotes();
           
            DictionaryManager.getManager().deleteAllLlenguesDiccionari();           
            
            // Esborrem els manteniments
            FontManager.getManager().deleteAllFonts();
            HyerarchyManager.getManager().deleteAllJerarquies();
            LanguageManager.getManager().deleteAllLlengues();
            MarkManager.getManager().deleteAllMarques();
            CategoryManager.getManager().deleteAllCategories();
            
            // Esborrem les fitxes
            DataManager.getManager().deleteAllFitxes();            
            
            // Esborrem el diccionari
            DictionaryManager.getManager().deleteAllDiccionari();
            
            // Fem els inserts necessaris de les jerarquies i marques de ponderacio 
            // En principi no cal ja que en el xml haura de venir la Jerarquia i la marca per defecte.
           HyerarchyManager.getManager().altaJerarquia("","","",0);
           MarkManager.getManager().altaMarca("","","");
    }
    
    // ----------------------------------------------------
    // Proces d'exportacio
    // ----------------------------------------------------
    
    //  Mtode encarregat de crear el XML d'exportaci en funci de la BD.
    public void createDictionaryXML(File file2export,String id) throws ExportException 
    {
        // Definicions dels elements i del document
        Element dadesDiccionari=null, dadesFitxes=null, dadesArbreCamp=null, dadesAreesPerFitxa=null, dadesLlengues=null,dadesDefinicions=null, dadesNotes=null, dadesContextos=null, dadesMarquesPonderacio=null, dadesCategories=null, dadesJerarquies=null, dadesDenominacions=null,dadesFonts=null;                
        Document doc=null;
        try
        {        

           // Creem el fitxer amb l'encoding 'UTF-8'
           ZipOutputStream exportFileOS= new ZipOutputStream(new FileOutputStream(file2export));
           exportFileOS.setLevel(9);
           ZipEntry entry = new ZipEntry("diccionari");
           
           exportFileOS.putNextEntry(entry);
           BufferedWriter outfileExport= new BufferedWriter(new OutputStreamWriter(exportFileOS,"UTF-8")); 
           
           // Indiquem la capalera del document xml.
           String initXml="<?xml version='1.0' encoding='UTF-8'?>";
           FunctionalModule.printStringInWriter(initXml,outfileExport);
                
           FunctionalModule.printStringInWriter("<Diccionari>",outfileExport);

            // Element Diccionari
            dadesDiccionari=DictionaryManager.getManager().getDadesDiccionariToXML(id);
            FunctionalModule.printElementInWriter(dadesDiccionari,outfileExport);
            
            // Element Fitxes
            dadesFitxes=DataManager.getManager().getAllFitxesToXML(id);
            FunctionalModule.printElementInWriter(dadesFitxes,outfileExport);
             
            // Element ArbreCamp
            dadesArbreCamp=AreaManager.getManager().getAllArbreCampsToXML(id);
            System.out.println("arbrecamp: "+dadesArbreCamp.toString());
            FunctionalModule.printElementInWriter(dadesArbreCamp,outfileExport);            
            
            // Element AreesFitxes
            dadesAreesPerFitxa=new Element("AreesPerFitxa");
            AreaManager.getManager().addAllAreesFitxesOfDictionary(dadesAreesPerFitxa,id);
            FunctionalModule.printElementInWriter(dadesAreesPerFitxa,outfileExport);            
            
            // Element Llengues 
            dadesLlengues=new Element("DadesLlengues");
            LanguageManager.getManager().addAllLlenguesToXML(dadesLlengues);
            LanguageManager.getManager().addAllLlenguesOfDictionaryToXML(id,dadesLlengues);
            FunctionalModule.printElementInWriter(dadesLlengues,outfileExport);  
            
            // Element Definicions
            dadesDefinicions=DefinitionManager.getManager().getAllDefinitionsToXML(id);
            FunctionalModule.printElementInWriter(dadesDefinicions,outfileExport);
                
            // Element Notes
            dadesNotes=NoteManager.getManager().getAllNotesToXML(id);
            FunctionalModule.printElementInWriter(dadesNotes,outfileExport);
                
            // Element Contextos
            dadesContextos=ContextManager.getManager().getAllContextesToXML(id);                    
            FunctionalModule.printElementInWriter(dadesContextos,outfileExport);

            // Element Marques de ponderacio
            dadesMarquesPonderacio=new Element("DadesMarques");
            MarkManager.getManager().addAllMarquesToXML(dadesMarquesPonderacio);
            FunctionalModule.printElementInWriter(dadesMarquesPonderacio,outfileExport);                
                
            // Element Categories
            dadesCategories=new Element("DadesCategories");
            CategoryManager.getManager().addAllCategoriesToXML(dadesCategories);
            FunctionalModule.printElementInWriter(dadesCategories,outfileExport);                

            // Element Jerarquies
            dadesJerarquies=new Element("DadesJerarquies");
            HyerarchyManager.getManager().addAllJerarquiesToXML(dadesJerarquies);
            FunctionalModule.printElementInWriter(dadesJerarquies,outfileExport);
            
            // Element Denominacions
            dadesDenominacions=DenominationManager.getManager().getAllDenominationsToXML(id);                     
            FunctionalModule.printElementInWriter(dadesDenominacions,outfileExport);                                          
            
            // Element Fonts
            FunctionalModule.printStringInWriter("<DadesFonts>",outfileExport);
            
            // Creaci dels subelements
            Element llistatFonts=new Element("LlistatFonts");
            FontManager.getManager().addAllFontsToXML(llistatFonts);
            FunctionalModule.printElementInWriter(llistatFonts,outfileExport);                   
            
            Element fontsDenominacions=new Element("DadesFontsDenominacions");
            FontManager.getManager().addAllFontsDenominacionsToXML(fontsDenominacions,id);
            FunctionalModule.printElementInWriter(fontsDenominacions,outfileExport);                   
            
            Element fontsDefinicions = new Element("DadesFontsDefinicions");
            FontManager.getManager().addAllFontsDefinicionsToXML(fontsDefinicions,id);
            FunctionalModule.printElementInWriter(fontsDefinicions,outfileExport);                   
            
            Element fontsContextes=new Element("DadesFontsContextes");
            FontManager.getManager().addAllFontsContextesToXML(fontsContextes,id);
            FunctionalModule.printElementInWriter(fontsContextes,outfileExport);                   
            
            Element fontsNotes=new Element("DadesFontsNotes");
            FontManager.getManager().addAllFontsNotesToXML(fontsNotes,id);
            FunctionalModule.printElementInWriter(fontsNotes,outfileExport);                                 

            // Tanquem el fitxer i les etiquetes que queden pendents.
            FunctionalModule.printStringInWriter("</DadesFonts>",outfileExport);                
            FunctionalModule.printStringInWriter("</Diccionari>",outfileExport);
            outfileExport.close();                
        }
        catch(Exception e)
        {
            e.printStackTrace();
            
            throw new ExportException(Literales.getTextForMessage("ExportKO"));   
        }
    }                      
}
