/*
 * FunctionalModule.java
 *
 * Created on 11 de julio de 2007, 13:09
 */

package com.termcat.layout;

import com.termcat.bd.beans.ArbreCamp;
import com.termcat.bd.exceptions.FinderException;
import com.termcat.bd.utils.DBConnect;
import com.termcat.business.areas.AreaManager;
import com.termcat.business.exceptions.AreaException;
import com.termcat.business.exceptions.CategoryException;
import com.termcat.business.exceptions.DictionaryLanguageException;
import com.termcat.business.exceptions.FontException;
import com.termcat.business.exceptions.HyerarchyException;
import com.termcat.business.exceptions.LanguageException;
import com.termcat.business.exceptions.MarkException;
import com.termcat.business.exceptions.SearchException;
import com.termcat.business.exceptions.ToolException;
import com.termcat.business.reports.veurefitxa.BeanVeureFitxa;
import com.termcat.business.reports.veurefitxa.BeanVeureFitxaDenominacio;
import com.termcat.layout.commons.TermcatFilter;
import com.termcat.layout.commons.dndtree.DNDTree;
import com.termcat.layout.commons.tree.DoubleClickTree;
import com.termcat.synchronizer.exceptions.ExportException;
import electric.xml.Element;
import java.awt.Dimension;
import java.awt.event.MouseListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Vector;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.table.DefaultTableModel;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import com.termcat.util.literales.Literales;
import com.termcat.layout.commons.lists.ItemListCustom;
import com.termcat.layout.commons.rtf.editor.RTFEditorKit;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 *
 * @author  Vazquezl
 */
public class FunctionalModule extends javax.swing.JPanel 
{
    // S'ha canviat a static per a poder ser retornat en getMainFrame()
    protected static MainFrame menuFrame;
    private JPanel mainPanel;
    
    // Constants
    protected final String LLENGUES_ASSOCIATE_TITLE=Literales.getTextForLiteral("AssociatedLang");
    protected final String LLENGUES_NO_ASSOCIATE_TITLE=Literales.getTextForLiteral("NoAssociatedLang");
    
    protected static final String FONTS_ASSOCIATE_TITLE=Literales.getTextForLiteral("AssociatedFont");
    protected static final String FONTS_NO_ASSOCIATE_TITLE=Literales.getTextForLiteral("NoAssociatedFont");
    
    protected final int DEPTH_MAX=10;
    
    private final String FK_MSG="[23000-";
    
    // Tamanys de les finestres 
    protected int HEIGHT_EDIT_FITXES;
    protected int WIDTH_EDIT_FITXES;
    
    protected int HEIGHT_ALTA_FITXES;
    protected int WIDTH_ALTA_FITXES;
    
    protected int HEIGHT_POPUP;
    protected int WIDTH_POPUP;


                
    //  Cal fer un constructor que noms faci el this.menuFrame=menuFrame pels Editor*.java que no cal que
    // inicialitzin els components???
    public FunctionalModule(MainFrame menuFrame) 
    {
        this.menuFrame = menuFrame;
        initComponents();
        Dimension dim=new Dimension(1000,750);
        this.setSize(dim);
        this.setPreferredSize(dim);
        this.setMinimumSize(dim);
        this.setMaximumSize(dim);
    }
    
    public JPanel getPanel()
    {
        return mainPanel;
    }

    public void setPanel (JPanel panel)
    {
        this.mainPanel = panel;
    }
    
    /*
     *      Metode que tractar les excepcions i/o errors produdes als mduls
     */
    protected void tratarError(Exception errorException)
    {
        String errorMsg=errorException.getMessage();
        
        String msg= Literales.getTextForMessage("Error");
        
        // Filtratge dels missatges d'error
        if (errorMsg==null)
            msg+=Literales.getTextForMessage("InternalError");
       else if ((errorException instanceof ToolException))
            msg+=errorMsg;         
        // Filtratge de les FK
        else if ((errorException instanceof FontException) && (errorMsg.contains(FK_MSG)))
            msg+=Literales.getTextForMessage("Font_ErrorFK");
        else if ((errorException instanceof MarkException) && (errorMsg.contains(FK_MSG)))
            msg+=Literales.getTextForMessage("Mark_ErrorFK");
        else if ((errorException instanceof HyerarchyException) && (errorMsg.contains(FK_MSG)))
            msg+=Literales.getTextForMessage("Hyerarchy_ErrorFK");
        else if ((errorException instanceof CategoryException) && (errorMsg.contains(FK_MSG)))
            msg+=Literales.getTextForMessage("Category_ErrorFK");        
        else if ((errorException instanceof DictionaryLanguageException) && (errorMsg.contains(FK_MSG)))
            msg+=Literales.getTextForMessage("DictionaryLanguage_ErrorFK");               
        else if ((errorException instanceof AreaException) && (errorMsg.contains(FK_MSG)))
            msg+=Literales.getTextForMessage("Arees_ErrorFK");
        else if ((errorException instanceof LanguageException) && (errorMsg.contains(FK_MSG)))
            msg+=Literales.getTextForMessage("Language_ErrorFK");
        else if (errorException instanceof SearchException)
            msg+=Literales.getTextForMessage("Search_Error");
        
        // Error genric.
        else
            msg+=Literales.getTextForMessage("InternalError");
        
        // Mostrem la finestra amb el missatge d'error corresponent.
        errorException.printStackTrace();
        menuFrame.showMessageError(msg);

    }
    
    protected boolean validateTextComponent(String name,JTextComponent textComp, boolean required, int maxSice)
    {
        boolean valid = true;
        String [] argumentos = {name,maxSice+""};
        String text = textComp.getText();
        if(required && text.length()==0){
            menuFrame.showMessageError(Literales.getTextForMessageWithArgs("CampoObligatorio",argumentos));
            valid = false;
        }
        else if(text.length()>maxSice){
            menuFrame.showMessageError(Literales.getTextForMessageWithArgs("TamanyoMaximoSuperado",argumentos));
            valid = false;
        }
        return valid;
    }
    
        
    protected Vector getNoAssociates(Vector all, Vector associates) 
    {
        String id=null;
        Vector res=new Vector();
        res=(Vector) all.clone();
        
        if (associates.size()>0)
        {
            ItemListCustom ci=new ItemListCustom();            
            for (int i=0;i<associates.size();++i)
            {
                ci=(ItemListCustom)associates.get(i);
                int pos=calculaPosition(ci.getId(), res);
                // Esborrem de la llista de tots aquells que estiguin al vector de associates. Aix el all, acabar
                // siguent el (all-associates). En cas que s'hagi esborrat de la taula de manteniments (calculatePosition
                // retornar un -1) llavors el que es fa es treure el valor de la llista d'associats, aix no apareixer
                // com a criteri de selecci del filtre.
                if(pos>=0)
                    res.remove(pos);
                else
                    associates.remove(i);
            }
        }
        return res;
    }

    // Funci que retorna la posici de l'objecte amb identificador id al vector v. En cas de no estar al vector,
    // retorna un -1.
    private int calculaPosition(String id, Vector v) 
    {
        int position=-1;
        ItemListCustom il=new ItemListCustom();
        for (int i=0;i<v.size();++i)
        {
            il=(ItemListCustom)v.get(i);
                       
            if(id.equals(il.getId()))
            {
                position=i;
                break;
            }
        }
        return position;
    }
    
    // Abans era protected, s'ha canviat perque desde VeureFitxaReportManager s'ha de fer un closeEditPanel()
    public static MainFrame getMainFrame()
    {
        return menuFrame;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
    // -------------------------------------------------------------------------------------------
    //       MTODES D'INICIALITZACI DE LES AREES TEMTIQUES, ESTA AQUI PERQUE S'UTILITZA TANT EN
    //      AREESMAIN.JAVA COM EN AREASTEMATIQUESALTA.JAVA
    // -------------------------------------------------------------------------------------------
    
    public JTree initJTree() 
    {          

        // Inicialitzem el node arrel de l'arbre
        ArbreCamp acRoot=new ArbreCamp();
        DefaultMutableTreeNode rootTreeNode=null;
        try 
        {
            acRoot=AreaManager.getManager().getRootNodeTree(this.menuFrame.getIdCurrentDictionary());
            rootTreeNode=new DefaultMutableTreeNode(acRoot);

            // Obtenim els fills i en cridem la funcio recursiva.
            List rootChildren=(List)AreaManager.getManager().getChildren(this.menuFrame.getIdCurrentDictionary(),acRoot.getIdArea());
            int rootListSize=rootChildren.size();
            if (rootChildren!=null && rootListSize>0)
            {
                for (int i=0;i<rootListSize;++i)
                    initTreeRecursive(this.menuFrame.getIdCurrentDictionary(),(ArbreCamp)rootChildren.get(i),rootTreeNode,1);
            }
        } 
        catch (AreaException ex) 
        {
            ex.printStackTrace();
        }
        return new DNDTree(rootTreeNode);
    }
     
    /*
     *       Mtode recursiu que s'encarrega de crear un arbre a partir d'un node('nodePare').
     *      @param idDiccionari:        Identificador del diccionari en el que estem treballant.
     *      @param ac:                  Objecte ArbreCamp que es vol afegir a l'arbre.
     *      @param nodePare:            Node del que penjara tot l'arbre que es creii.
     *      @param profunditat:         Nivell de profunditat en el que es troba l'objecte ArbreCamp que es vol afegir a l'arbre.
     */
    private void initTreeRecursive(String idDiccionari,ArbreCamp ac, DefaultMutableTreeNode nodePare, int profunditat)
    {
        if (profunditat>DEPTH_MAX)
        {
            // Error
        }
        else
        {   
            // Creem un nou node amb l'arbreCamp que ens passen per parmetre i l'afegim al pare.
            DefaultMutableTreeNode item=new DefaultMutableTreeNode(ac);   
            nodePare.add(item);
            
            //  Obtenim els fills del node. En cas de tenir-ne es crida a si mateix per a cadascun dels fills amb un nivell
            // mes de profunditat.
            List children=(List)AreaManager.getManager().getChildren(idDiccionari,ac.getIdArea());
            int childrenSize=children.size();
            if (children!=null && childrenSize>0)
            {
                for (int i=0;i<childrenSize;++i)
                    initTreeRecursive(idDiccionari,(ArbreCamp)children.get(i),item,profunditat+1);
            }
        } 
    }
    
    /*  
     *       Mtode que calcula el nom absolut d'una rea temtica. Es defineix aqui perque s'utilitza tant en el Fitxes Main
     *      com en la classe Fitxa.
     *      @param idPare:      Identificador del pare.
     *      @param nomArea:     Nom de l'rea temtica.
     *      @return:            Nom absolut de l'rea temtica.
     */
    public static String getAbsoluteAreaName(String idPare, String nomArea, String idArea) 
    {
        /*String nomAbsolut="";
        if(idPare==null) return nomAbsolut; 
        try
        {
             if("root".equals(idArea))
             {
                 nomAbsolut= nomArea;
             }
             else
             {
                    ArbreCamp arbreCamp= new ArbreCamp();
                    ArbreCamp arbrePare=(ArbreCamp) arbreCamp.findByPrimaryKey(idPare);
                    String nomAreaPare=arbrePare.getNomArea();
                    nomAbsolut = getAbsoluteAreaName(arbrePare.getIdPare(),nomAreaPare,arbrePare.getIdArea()) + " / " + nomArea;
             }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }*/
        
        // Com que ara no volen que es mostri tot l'arbre, fem un return del nomArea.
        return nomArea;
    }

    // Es fa aqui perque AreesTematiquesAlta necessita la inicialitzaci i no poc extendres de Fitxa ja que
    // ha de mostrar errors per pantalla.(tratarError)
     protected JTable initTable(String titol, String idFitxa) 
    {
         String nomAreaAbsolut=null;
         ArbreCamp ac=new ArbreCamp();
         JTable table=null;
         
         // Mirar si en el cas que names sigui nul no crea la taula del titol.           
         final String[] names = {"idArea",titol};       
         final Object[][] data =null;

         DefaultTableModel model=new DefaultTableModel(data,names);       
         try
         {
             // Obtenim les arees temtiques de la fitxa
             Vector areesTematiques= new Vector();
             areesTematiques=AreaManager.getManager().getAreesOfFitxa(idFitxa);
             if (areesTematiques.size()==0)
                 model.addRow(new Object[] {null,null});        
             else
             {
                 for (int i=0;i<areesTematiques.size();++i)
                 {
                    ac=(ArbreCamp) areesTematiques.get(i);
                    nomAreaAbsolut=getAbsoluteAreaName(ac.getIdPare(),ac.getNomArea(),ac.getIdArea());
                    model.addRow(new Object[] {ac.getIdArea(),nomAreaAbsolut});        

                  }    
             }

             table=new JTable(model)
             {
                public boolean isCellEditable(int row, int column)
                {
                        return false;
                }
            };
             table.getColumnModel().removeColumn(table.getColumnModel().getColumn(0));
         }
         catch(Exception e)
         {
             
         }
         return table;
    }

    private int getCodeLast(String errorMsg) 
    {
        return 30;
    }

    private int getCodeFirst(String errorMsg) {
        return 50;
    }

    // Es posa aqui perque s'utilitza en el FitxesMain com en el EditorFitxes.
     protected void actionWindowClose(java.awt.event.WindowEvent evt, boolean isAlta) 
     {
          try
          {
            DBConnect.getConnectionTransactional().rollback();
            DBConnect.putConnectionTransactional();
            this.menuFrame.setCurrentFitxaDirty(false);
            this.menuFrame.setIdFitxa(null);
            
            if (isAlta)
                this.menuFrame.closeAltaPanelPopup();
            else
                this.menuFrame.closeEditPanelPopup();
          }
          catch (Exception e)
          {
              e.printStackTrace();
          }
     }
     
    /*       Mtode que habilita/deshabilita els botons d'augmentar i baixar l'ordre de les categories en funci 
     *      de la fila indicada.
     *      @param rowSelected:     Fila de la taula. En cas que el valor sigui -1 es busca quina
     *                             es la fila que est seleccionada de la taula.
     */    
    protected void setEnableButtons(JTable table, int rowSelected, JButton ordreUp, JButton ordreDown) 
    {   
        int row=-1;

        // Obtenim el numero de files de la taula.
        int maxRow=table.getRowCount();
        
        if (rowSelected==-1)
            row=table.getSelectedRow();
        else
            row=rowSelected;
        
        // Cas en que la taula te com a mxim una nica fila.  S'inhabilita els dos botons.
        if (maxRow<=1)
        {
            ordreDown.setEnabled(false);
            ordreUp.setEnabled(false);
        }    
        else
        {
            // La fila es la primera, per tant no es pot pujar.  S'inhabilita el bot de pujar.
            if (row==0)        
            {
                ordreUp.setEnabled(false);
                ordreDown.setEnabled(true);
            }
            // La fila es la ltima, per tant no es pot baixar. S'inhabilita el bot de baixar.
            if (row==maxRow-1)
            {
                ordreDown.setEnabled(false);
                ordreUp.setEnabled(true);
            }
            // Cas que la fila no sigui ni la primera ni la ltima, s'habiliten els dos botons.
            if (row>0 && row<maxRow-1)
            {
                ordreDown.setEnabled(true);
                ordreUp.setEnabled(true);                
            }
        }         
    }
    
    protected File getFiles() 
    {
        String fileString;
        File fileName;
        // Display file dialog so user can select file to open
        JFileChooser fileChooser = new JFileChooser();
        TermcatFilter filter = new TermcatFilter();
        fileChooser.setFileFilter(filter);        
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

        int result = fileChooser.showOpenDialog(this);

        // If cancel button selected return
        if (result == JFileChooser.CANCEL_OPTION)
        {
            return null;
        }
        
        // Obtain selected file
        fileName = fileChooser.getSelectedFile();
        
        return fileName;
    }

    public JRadioButton getSelectedRadioButton(ButtonGroup group) 
    {
        for (Enumeration e=group.getElements(); e.hasMoreElements(); ) 
        {
            JRadioButton b = (JRadioButton)e.nextElement();
            if (b.getModel() == group.getSelection())             
                return b;
        }
        return null;
    }

    //  Mtode que s'encarrega d'esborrar els anteriors Listeners de la classe DoubleClickTree
    public void removeAll2ClickListeners(JTree tree) 
    {
        MouseListener[] mls=tree.getMouseListeners();
        for (int i=0;i<mls.length;++i)
        {
            if (((MouseListener)mls[i]) instanceof DoubleClickTree)
            {
                tree.removeMouseListener(mls[i]);
            }
        }
    }
    public static String convertRTF2Text(String rtfText) throws Exception
    {
        String s ="";
        if(rtfText!=null && !rtfText.equals(""))
        {
            StringReader sr = new StringReader(rtfText);

            JEditorPane editor = new JEditorPane();
            RTFEditorKit kit = new RTFEditorKit();
            editor.setEditorKit(kit);
            Document doc = editor.getDocument();

            kit.read(sr, doc, 0);
            s = doc.getText(0, doc.getLength());
        }
        return s;
    }

    /*
     *       Mtode encarregat de crear el format per a les fonts en la opci de 'Veure Fitxa'.
     *      Se li passa com a parmetres un vector amb totes les 
     */
    public static Vector<BeanVeureFitxa> createListFonts(Vector<BeanVeureFitxa> vIni)
    {
        Vector<BeanVeureFitxa> vOut=new Vector<BeanVeureFitxa>();
        String font="";
        boolean isAdded=false;
        if (vIni!=null && vIni.size()>0)
        {
            // Inicialitzem les variables
            String lastId=vIni.get(0).getId();
            String fontIni=vIni.get(0).getFonts();
            if (fontIni==null) fontIni=" ";
            String fonts="("+fontIni;

            for(int i=1;i<vIni.size();++i)
            {            
                font=vIni.get(i).getFonts();
                if(lastId.equals(vIni.get(i).getId()))
                {
                    if (font!=null)
                        fonts+=";"+font;            
                    isAdded=false;
                }
                else
                {
                    // Tanquem la font i afegim el Bean anterior (amb el camp fonts en el format correcte) al vector de sortida.  
                    fonts+=")";
                    if (fonts.equals("( )")) fonts="";                    
                    vIni.get(i-1).setFonts(fonts);
                    
                    vOut.add(vIni.get(i-1));                   

                    // Iniciem la seguent denominacio
                    if (font==null) font=" ";
                    fonts="("+font;                    
                    isAdded=(i<vIni.size()-1);

                    // Actualitzem el valor del lastId
                    lastId=vIni.get(i).getId();                    
                }
            }
            // En el cas que l'ultim id sigui igual l'afegim al vector.
            if(!isAdded)
            {
               
                fonts+=")";
                if (fonts.equals("( )")) fonts="";                
                vIni.get(vIni.size()-1).setFonts(fonts);                
               
                vOut.add(vIni.get(vIni.size()-1));
            }                            
        }            
        return vOut;
    }
    
    public static Vector<BeanVeureFitxaDenominacio> createFontDenominacions(Vector<BeanVeureFitxaDenominacio> vIni)
    {
        Vector<BeanVeureFitxaDenominacio> vOut=new Vector<BeanVeureFitxaDenominacio>();
                        
        String font="";
        boolean isAdded=false;
        if (vIni!=null && vIni.size()>0)
        {
            // Inicialitzem les variables
            String lastId=vIni.get(0).getId();
            String fontIni=vIni.get(0).getFonts();
            if(fontIni==null) fontIni=" ";
            String fonts="("+fontIni;

            for(int i=1;i<vIni.size();++i)
            {            
                font=vIni.get(i).getFonts();
                if(lastId.equals(vIni.get(i).getId()))
                {
                    if(font!=null)
                        fonts+=";"+font;            
                    isAdded=false;
                }
                else
                {
                    // Tanquem la font i afegim el Bean anterior (amb el camp fonts en el format correcte) al vector de sortida.
                    fonts+=")";
                    if (fonts.equals("( )")) fonts="";
                    vIni.get(i-1).setFonts(fonts);

                    vOut.add(vIni.get(i-1));
                                        
                    // Iniciem la seguent denominacio
                    if (font==null) font=" ";
                    fonts="("+font;                    
                    isAdded=(i<vIni.size()-1);
                    
                    // Actualitzem el valor del lastId
                    lastId=vIni.get(i).getId();                    
                }
            }
            // En el cas que l'ultim id sigui igual l'afegim al vector.
            if(!isAdded)
            {
                fonts+=")";
                if (fonts.equals("( )")) fonts="";                
                vIni.get(vIni.size()-1).setFonts(fonts);                

                vOut.add(vIni.get(vIni.size()-1));

                //#IP printem per pantalla
                System.out.println("LAST font["+(vIni.size()-1)+"]"+fonts);                                    
                System.out.println("LAST Denominacio revisat: "+vIni.get(vIni.size()-1).getDenominacio());                                  
            }                            
        }            
                
        return vOut;
    }

    // Mtode per imprimir un string a un bufferedWriter
    public static  void printStringInWriter(String initStr,BufferedWriter outfile) throws IOException         
    {
        outfile.append(initStr);
        outfile.newLine();
    }

   // Mtode per imprimir un element a un bufferedWriter
    public static void printElementInWriter(Element e, BufferedWriter outfile) throws IOException 
    {
        // En cas que l'element estigui buit, que es fa?
        if (e!=null)
        {
            String elemAsString=e.toString();                
            outfile.append(elemAsString);
            outfile.newLine();
        }
    } 
    
    public JDialog initLoader() 
    {                          
        JDialog loader;                                
        JOptionPane pane = new JOptionPane(Literales.getTextForMessage("WaitMessage"),JOptionPane.INFORMATION_MESSAGE);
        
        try
        {
             ImageIcon icon = new ImageIcon(this.getClass().getResource("/com/termcat/resources/img/loader.gif"));
             pane.setIcon(icon);
        }
        catch(Exception e){}
                        
        loader = pane.createDialog(null,"");
        loader.setBounds(350,330,305,140);
        loader.setModal(false);        
        
        return loader;
    }
  
}
